<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once ABSPATH . 'wp-admin/includes/class-wp-list-table.php';
}


class Woocommerce_Inbank_Payment_List extends WP_List_Table {

	/**
	 * Max items.
	 *
	 * @var int
	 */
	protected $max_items;

	/**
	 * Constructor.
	 */
	public function __construct() {

		parent::__construct(
			array(
				'singular' => __( 'Payment', 'woocommerce-inbank'),
				'plural'   => __( 'Payments', 'woocommerce-inbank'),
				'ajax'     => false,
			)
		);
	}

	/**
	 * No items found text.
	 */
	public function no_items() {
		echo __( 'No payments added.', 'woocommerce-inbank' );
	}

    /**
	 * Get column value.
	 *
	 * @param mixed  $item
	 * @param string $column_name
	 */
	public function column_default( $item, $column_name )
    {
        $class_name = $item['class_name'];
		switch ( $column_name ) {
			case 'product_code':
			    return $item [ $column_name ];
			case 'product_name':
                return $class_name::getDefaultTitle();
			case 'country':
                return WC()->countries->get_countries()[$item [ $column_name ]];
			case 'title':
                return self::get_payment_title($item['id_payment']);
            case 'enabled':
                return (int)$item[ $column_name ] == 1 ? __('Yes', 'woocommerce-inbank') : __('No', 'woocommerce-inbank');
            case 'actions':
				?><p>
					<?php
					$actions   = array();
					$id_payment = $item['id_payment'];

					$actions['edit'] = array(
						'url'    => admin_url( 'options-general.php?page=woocommerce-inbank-settings&action=edit_payment&id_payment=' . $id_payment ),
						'name'   => __( 'Edit', 'woocommerce-inbank' ),
						'action' => 'edit',
					);

					$actions['enable_disable'] = array(
						'url'    => admin_url( 'options-general.php?page=woocommerce-inbank-settings&action=enable_disable_payment&id_payment=' . $id_payment ),
						'name'   => ((int)$item['enabled'] > 0 ? __( 'Disable', 'woocommerce-inbank' ) : __( 'Enable', 'woocommerce-inbank' )),
						'action' => 'enable_disable',
					);

					$actions['delete'] = array(
						'url'    => admin_url( 'options-general.php?page=woocommerce-inbank-settings&action=delete_payment&id_payment=' . $id_payment ),
						'name'   => __( 'Delete', 'woocommerce-inbank' ),
						'action' => 'delete',
                        'onclick' => 'return confirm(\''.__('Are you sure you want to delete payment method').'?\')'
					);


					foreach ( $actions as $action ) {
						printf(
							'<a class="button tips %1$s" href="%2$s" data-tip="%3$s" style="margin-right: 10px" onclick="%5$s">%4$s</a>',
							esc_attr( $action['action'] ),
							esc_url( $action['url'] ),
							sprintf( esc_attr__( '%s payment', 'woocommerce-inbank' ), $action['name'] ),
							esc_html( $action['name'] ),
                            (isset($action['onclick']) ? $action['onclick'] : '')
						);
					}
					?>
				</p>
				<?php
				break;
		}
	}

	/**
	 * Get columns.
	 *
	 * @return array
	 */
	public function get_columns() {

		$columns = array(
			'product_name'       => __( 'Product', 'woocommerce-inbank' ),
			'product_code'  => __( 'Product code', 'woocommerce-inbank' ),
			'country'  => __( 'Country', 'woocommerce-inbank' ),
			'title'  => __( 'Payment title', 'woocommerce-inbank' ),
			'enabled'   => __( 'Enabled', 'woocommerce-inbank' ),
			'actions'   => __( 'Actions', 'woocommerce-inbank' ),
		);

		return $columns;
	}

	/**
	 * Prepare customer list items.
	 */
	public function prepare_items() {

		$this->_column_headers = array( $this->get_columns(), array(), $this->get_sortable_columns() );
		$current_page          = absint( $this->get_pagenum() );
		$per_page              = 999;

		$this->get_items( $current_page, $per_page );

		/**
		 * Pagination.
		 */
		$this->set_pagination_args(
			array(
				'total_items' => $this->max_items,
				'per_page'    => $per_page,
				'total_pages' => ceil( $this->max_items / $per_page ),
			)
		);

        $this->items = self::get_payments( $per_page, $current_page );

    }

    /**
     * Retrieve customers data from the database
     *
     * @param int $per_page
     * @param int $page_number
     *
     * @return mixed
     */
    public static function get_payments( $per_page = 5, $page_number = 1 ) {

        global $wpdb;

        $sql = "SELECT ip.*, COALESCE(NULLIF(ip.product_code, ''), ip.product_code_debug) AS product_code FROM {$wpdb->prefix}inbank_payment ip";

        $sql .= " LIMIT $per_page";
        $sql .= ' OFFSET ' . ( $page_number - 1 ) * $per_page;


        $result = $wpdb->get_results( $sql, 'ARRAY_A' );
        if (!empty($result)) {
            foreach ($result as $idx => $row)
                if (!empty($row['class_name']) && !class_exists($row['class_name']))
                    unset($result[$idx]);
        }

        return $result;
    }

    public static function get_payment_title($id_payment, $lang = null)
    {
        global $wpdb;

        if (empty($lang))
            $lang = strtoupper(InbankFrameworkBridge::getLocaleIso());

        $sql = "SELECT custom_title FROM {$wpdb->prefix}inbank_payment_lang WHERE id_payment = ".$id_payment." AND lang = '".$lang."'";

        $result = $wpdb->get_var( $sql );

        return $result;
    }
}
