;(function ( $, window, document ) {
	'use strict';

	$(document).on('change', '#country', function () {
		insertParam('country', this.value);
	});

	$(document).on('change', '#debug_mode', function () {
		switchDebugLive();
	});

	switchDebugLive();
	langFieldSwitcher();
	hookTabs();
	hookListActions();
	initProductSelector();

	function insertParam(key, value) {
		key = encodeURIComponent(key);
		value = encodeURIComponent(value);

		// kvp looks like ['key1=value1', 'key2=value2', ...]
		var kvp = document.location.search.substr(1).split('&');
		let i=0;

		for(; i<kvp.length; i++){
			if (kvp[i].startsWith(key + '=')) {
				let pair = kvp[i].split('=');
				pair[1] = value;
				kvp[i] = pair.join('=');
				break;
			}
		}

		if(i >= kvp.length){
			kvp[kvp.length] = [key,value].join('=');
		}

		// can return this or...
		let params = kvp.join('&');

		// reload page with new params
		document.location.search = params;
	}

	function switchDebugLive()
	{
		var debug_mode = $('#debug_mode');

		if (debug_mode.length > 0)
		{
			var debug_mode_val = debug_mode.val();

			if (debug_mode_val == 1)
			{
				$('#product_code_debug').closest('tr').show();
				$('#api_key_debug').closest('tr').show();
				$('#shop_uuid_debug').closest('tr').show();

				$('#product_code').closest('tr').hide();
				$('#api_key').closest('tr').hide();
				$('#shop_uuid').closest('tr').hide();
			}
			else
			{
				$('#product_code_debug').closest('tr').hide();
				$('#api_key_debug').closest('tr').hide();
				$('#shop_uuid_debug').closest('tr').hide();

				$('#product_code').closest('tr').show();
				$('#api_key').closest('tr').show();
				$('#shop_uuid').closest('tr').show();

			}
		}
	}

	function langFieldSwitcher()
	{
		var lang_select = $('select.lang_selector');
		if (lang_select.length > 0)
		{
			$(document).on('change', 'select.lang_selector', function() {
				var input_field_id = $(this).data('input_field_id');
				var desc_field_id = $(this).data('desc_field_id');
				var selected_lang = $(this).val();

				// inputs
				$('[name^="'+input_field_id+'"]').each(function () {
					$(this).hide();
				});
				$('[name="'+input_field_id+'['+selected_lang+']"').show();

				// inputs desc
				$('[id^="'+desc_field_id+'_"][id$="_desc"]').each(function () {
					$(this).hide();
				});
				$('#'+desc_field_id+'_'+selected_lang+'_desc').show();
			});
		}
	}

	function hookTabs() {
		$(document).find('.categorydiv').each(function () {
			var this_id = $(this).attr('id'), taxonomyParts, taxonomy, settingName;

			taxonomyParts = this_id.split('-');
			taxonomyParts.shift();
			taxonomy = taxonomyParts.join('-');
			settingName = taxonomy + '_tab';

			if (taxonomy == 'category') {
				settingName = 'cats';
			}

			$('a', '#' + taxonomy + '-tabs').on('click', function (e) {
				e.preventDefault();
				var t = $(this).attr('href');
				$(this).parent().addClass('tabs').siblings('li').removeClass('tabs');
				$('#' + taxonomy + '-tabs').siblings('.tabs-panel').hide();
				$(t).show();
				if ('#' + taxonomy + '-all' == t) {
					deleteUserSetting(settingName);
				} else {
					setUserSetting(settingName, 'pop');
				}
			});

			if (getUserSetting(settingName))
				$('a[href="#' + taxonomy + '-pop"]', '#' + taxonomy + '-tabs').trigger('click');


			$('#' + taxonomy + 'checklist, #' + taxonomy + 'checklist-pop').on(
				'click',
				'li.popular-category > label input[type="checkbox"]',
				function () {
					var t = $(this), c = t.is(':checked'), id = t.val();
					if (id && t.parents('#taxonomy-' + taxonomy).length)
						$('#in-' + taxonomy + '-' + id + ', #in-popular-' + taxonomy + '-' + id).prop('checked', c);
				}
			);

		});
	}

	function template(data) {
		if (data.img_url === 'undefined')
		{
			return $("<span>" + data.title + "</span>")
		}
		else
		{
			return $("<span style='display: flex; align-content: center;'><img src='" + data.img_url + "' style='height: 25px; padding-right: 10px'/>" + data.title + "</span>")
		}
	}

	function hookListActions() {
		$(document).on('click', '.selected_product_list .dashicons-trash', function (e){
			$(this).parent().remove();
		});
	}

	function initProductSelector()
	{
		if ($(document).find('.inbank_product_select').length > 0)
		{
			var select_input = $(document).find('.inbank_product_select');
			select_input.select2(
				{
					ajax: {
						url: ajaxurl, // AJAX URL is predefined in WordPress admin
						dataType: 'json',
						delay: 250, // delay in ms while typing when to perform a AJAX search
						data: function (params) {
							return {
								q: params.term, // search query
								action: 'inbank_get_products', // AJAX action for admin-ajax.php
								exclude_ids: jQuery('.selected_product_list').find('[name="woocommerce-inbank-lt_restrictions[selected_products][]"]').map(function(){return this.value}).get().join()
							};
						},
						processResults: function (data, params) {
							return {
								results: $.map(data, function (item) {
									return {
										text: template(item),
										id: item.id_product,
										data: item,
									};
								})
							};
						},
					},
					cache: true,
					allowClear: true,
					placeholder: 'Please choose product',
					minimumInputLength: 3 // the minimum of symbols to input before perform a search
				}
			);

			select_input.on("select2:select", function (e) {
				select_input.val('').trigger('change'); // Change the value or make some change to the internal state
				select_input.trigger('change.select2');

				var li_template = $('.selected_product_list_item_template').html();
				var data = e.params.data.data;
				var new_item = li_template.replace('[id_product]', data.id_product).replace('[img_url]', data.img_url).replace('[title]', data.title).replace('data-src', 'src').replace('disabled', '');
				$('.selected_product_list').append(new_item);
			});
		}
	}

})( jQuery, window, document );
