<?php
/**
 * Plugin Updater
 *
 * @link       https://inbank.ee
 * @since      1.0.0
 *
 * @package    Woocommerce_Inbank
 * @subpackage Woocommerce_Inbank/includes
 */

/**
 * Fired during plugin update.
 *
 * This class defines all code necessary to run during the plugin's update.
 *
 * @since      1.0.0
 * @package    Woocommerce_Inbank
 * @subpackage Woocommerce_Inbank/includes
 * @author     Inbank
 */
class Woocommerce_Inbank_Updater
{
    private static $previous_version;


    public static function update()
    {
        self::$previous_version = get_option('woocommerce-inbank_last_update_check_version', '1.0.0');
        if (version_compare(self::$previous_version, WOOCOMMERCE_INBANK_VERSION, '<'))
        {
            // update actions
            self::update_to_2_0_3();
            self::update_to_2_0_4();
            self::update_to_2_0_7();
            self::update_to_2_0_9();
            self::update_to_2_1_2();

            update_option('woocommerce-inbank_last_update_check_version', WOOCOMMERCE_INBANK_VERSION);
        }
    }

    private static function update_to_2_0_3()
    {
        if (version_compare(self::$previous_version, '2.0.3', '<'))
        {
            global $wpdb;
            $inbank_payment_table_name = $wpdb->prefix . "inbank_payment";

            $alter_table_sql = "ALTER TABLE `{$inbank_payment_table_name}` ADD `api_last_upd` DATETIME";
            try {
                $wpdb->query($alter_table_sql);
            }
            catch (Exception $ex) {}
        }
    }

    private static function update_to_2_0_4()
    {
        if (version_compare(self::$previous_version, '2.0.4', '<'))
        {
            global $wpdb;
            $inbank_payment_table_name = $wpdb->prefix . "inbank_payment";

            $alter_table_sql = "ALTER TABLE `{$inbank_payment_table_name}` ADD `custom_logo` VARCHAR(255)";
            try {
                $wpdb->query($alter_table_sql);
            }
            catch (Exception $ex) {}
        }
    }

    private static function update_to_2_0_7()
    {
        if (version_compare(self::$previous_version, '2.0.7', '<'))
        {
            global $wpdb;
            $inbank_payment_table_name = $wpdb->prefix . "inbank_payment";

            $alter_table_sql = "ALTER TABLE `{$inbank_payment_table_name}` MODIFY COLUMN `period` VARCHAR(512)";
            try {
                $wpdb->query($alter_table_sql);
            }
            catch (Exception $ex) {}
        }
    }

    private static function update_to_2_0_9()
    {
        if (version_compare(self::$previous_version, '2.0.9', '<'))
        {
            global $wpdb;
            $inbank_payment_table_name = $wpdb->prefix . "inbank_payment";

            $alter_table_sql = "ALTER TABLE `{$inbank_payment_table_name}` ADD `restrictions_selected_categories` VARCHAR(512)";
            try {
                $wpdb->query($alter_table_sql);
            }
            catch (Exception $ex) {}

            $alter_table_sql = "ALTER TABLE `{$inbank_payment_table_name}` ADD `restrictions_selected_products` VARCHAR(512)";
            try {
                $wpdb->query($alter_table_sql);
            }
            catch (Exception $ex) {}

            $alter_table_sql = "ALTER TABLE `{$inbank_payment_table_name}` ADD `restrictions_condition` VARCHAR(10)";
            try {
                $wpdb->query($alter_table_sql);
            }
            catch (Exception $ex) {}
        }
    }

    private static function update_to_2_1_2()
    {
        if (version_compare(self::$previous_version, '2.1.2', '<'))
        {
            global $wpdb;

            try {
                $sql = "DELETE id FROM ".$wpdb->prefix."inbank_detail id
                                                            INNER JOIN (SELECT id_order, MAX(id_detail) AS id_detail 
                                                            FROM ".$wpdb->prefix."inbank_detail GROUP BY id_order HAVING COUNT(1) > 1) tbl
                                                            ON tbl.id_order = id.id_order AND id.id_detail <> tbl.id_detail";
                $wpdb->query($sql);
            }
            catch (Exception $ex) {}


            $alter_table_sql = "ALTER TABLE ".$wpdb->prefix."inbank_detail ADD UNIQUE INDEX `UNIQUE` (id_order);";
            try {
                $wpdb->query($alter_table_sql);
            }
            catch (Exception $ex) {}
        }
    }
}
