<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://inbank.ee
 * @since      1.0.0
 *
 * @package    Woocommerce_Inbank
 * @subpackage Woocommerce_Inbank/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Woocommerce_Inbank
 * @subpackage Woocommerce_Inbank/public
 * @author     Inbank
 */
class Woocommerce_Inbank_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $woocommerce_inbank    The ID of this plugin.
	 */
	private $woocommerce_inbank;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $woocommerce_inbank       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $woocommerce_inbank, $version ) {

		$this->woocommerce_inbank = $woocommerce_inbank;
		$this->version = $version;

	}

    /**
     * Enqueue Javascript files.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {
        if ( function_exists('is_product') && is_product() )
        {
            if (INBANK_USE_TIPPY)
            {
                wp_enqueue_script($this->woocommerce_inbank.'_pooper',
                    plugin_dir_url(__FILE__) . 'js/popper.min.js', [], $this->version,
                    true);
                wp_enqueue_script($this->woocommerce_inbank.'_tippy',
                    plugin_dir_url(__FILE__) . 'js/tippy-bundle.umd.min.js', [], $this->version,
                    true);

                wp_add_inline_script($this->woocommerce_inbank.'_tippy', sprintf( "setTimeout(() => { tippy('[data-tippy-content]:not([data-tippy-content=\"\"])'); }, 2000);" ) );
            }

            //wp_enqueue_script($this->woocommerce_inbank,
            //    plugin_dir_url(__FILE__) . 'js/woocommerce-inbank-public.js', array('jquery'), $this->version,
            //    false);
        }
    }

    /**
     * Enqueue CSS files.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {
        if ( function_exists('is_product') && is_product() ) {
            wp_enqueue_style($this->woocommerce_inbank.'_product',
                plugin_dir_url(__FILE__) . 'css/woocommerce-inbank-product.css', array(), $this->version, 'all');
        }

        if ( function_exists('is_checkout') && is_checkout() ) {
            wp_enqueue_style($this->woocommerce_inbank.'_checkout',
                plugin_dir_url(__FILE__) . 'css/woocommerce-inbank-checkout.css', array(), $this->version, 'all');
        }
    }
}
