<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Wordpress to newer
 * versions in the future. If you wish to customize Wordpress for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

abstract class AbstractPayment
{

    public $object;
    public $production_api;
    public $debug_api;
    public $production_epos;
    public $debug_epos;
    public $has_large_calc = false;
    public $product_code;
    public $default_logo_name;
    public $subtype;
    const INBANK_CALC_SMALL = 'calc_small';
    const INBANK_CALC_SMALL_DOWNPAYMENT = 'calc_small_w_downpayment';
    const INBANK_CALC_LARGE = 'calc_large';
    const INBANK_PAYMENT = 'payment';
    const INBANK_DOWNPAYMENT = 'downpayment';

    /**
     * AbstractPayment constructor.
     * @param $object_model array|InbankObjectModel
     */
    abstract public function __construct($object_model);

    public function getLogoPath($payment = false)
    {
        $style = '';
        $logo_size = $this->object->logo_size;

        if (!empty($this->object->custom_logo))
            $icon_url = wp_get_attachment_image_url($this->object->custom_logo);

        if (!isset($icon_url) || empty($icon_url))
        {
            $dir = INBANK_MODULE_URL.'assets/images/';
            if (empty($logo_size)) {
                $dir .= 'small_';
            } else {
                $dir .= $logo_size.'_';
            }

            if (is_string($this->default_logo_name)) {
                $icon_url = $dir.$this->default_logo_name;
            } else {
                $iso_code = strtolower(InbankFrameworkBridge::getLocaleIso());

                if (isset($this->default_logo_name[$iso_code])) {
                    $icon_url = $dir.$this->default_logo_name[$iso_code];
                } else {
                    $icon_url = $dir.reset($this->default_logo_name);
                }
            }

            $style = 'height: '.($logo_size == 'small' ? 26 : 36).'px;';
        }
        else
        {
            $style = 'max-height: '.(empty($logo_size) || $logo_size == 'small' ? 26 : 36).'px; margin-right: 10px;';
        }

        return [
            'url' => $icon_url,
            'style' => $style,
            'logo_size' => empty($logo_size) ? 'small' : $logo_size
        ];
    }

    /**
     * Used to get available payment product currencies
     * @return string[]
     */
    public static function getAvailableCurrencies()
    {
        return ['EUR'];
    }

    /*
     * Used this to get api url. Respects user selected mode.
     */
    public function getApiUrl()
    {
        if ($this->object->debug_mode == 1) {
            return $this->debug_api;
        } else {
            return $this->production_api;
        }
    }

    public function getEposUrl()
    {
        if ($this->object->debug_mode == 1) {
            return $this->debug_epos;
        } else {
            return $this->production_epos;
        }
    }

    public function getShopUUID()
    {
        if ($this->object->debug_mode == 1) {
            return $this->object->shop_uuid_debug;
        } else {
            return $this->object->shop_uuid;
        }
    }

    public function getApiKey()
    {
        if ($this->object->debug_mode == 1) {
            return $this->object->api_key_debug;
        } else {
            return $this->object->api_key;
        }
    }

    public function getProductCode()
    {
        if ($this->object->debug_mode == 1) {
            return $this->object->product_code_debug;
        } else {
            return $this->object->product_code;
        }
    }

    /** replaces placeholders {placeholder} with supplied array ['placeholder' => 'value']
     * @param $template string
     * @param $placeholders array
     * @return string
     */
    public static function replacePlaceholders($template, $placeholders)
    {
        $keys = [];
        $values = [];
        foreach ($placeholders as $key => $value) {
            $keys[] = "{".$key."}";
            $values[] = $value;
        }

        return str_replace($keys, $values, $template);
    }

    /**
     * @param $id_order
     * @param $message
     * @param  string  $type  error type either 'error' or 'warning'
     * @param  string  $product_code
     * @param  string  $endpoint
     * @param  string  $response
     */
    public static function addLog(
        $id_order,
        $message,
        $type = 'error',
        $product_code = "",
        $endpoint = "",
        $response = ""
    ) {
        InbankLogs::addLog($id_order, $message, $type, $product_code, $endpoint, $response);
    }


    /*
     * Product specific parameters
     */

    /**
     * Returns Product settings form suitable for Wordpress formbuilder
     * @return array[]
     */
    abstract public function productApiForm();

    /**
     * Returns default product title
     * @return string
     */
    abstract public static function getDefaultTitle();

    /**
     * Returns available country ID
     * @return int
     */
    abstract public static function getAvailableCountries();

    /**
     * Returns template vars
     * @param $template
     * @param  array  $additional_info
     * @return array
     */
    abstract public static function getPhraseTemplateVars($template, $additional_info = []);

    /**
     * Returns payment template
     * @param $product_price
     * @return string|false
     */
    abstract public function getPaymentTemplate($product_price);

    /**
     * Uses product price and returns small calculator template
     * @param $product_price
     * @return string|false
     */
    abstract public function getCalcSmallTemplate($product_price);

    /**
     * Uses product price and returns large calculator template
     * @param $product_price
     * @return false|string
     */
    abstract public function getCalcLargeTemplate($product_price);

    /**
     * Handle customer payments.
     * @param $id_order int Order object id
     */
    abstract public function handlePayment($id_order);

    /**
     *
     * @param $product_price
     * @param $currency_code
     * @return mixed
     */
    abstract public function renderPaymentForCheckout($product_price, $currency_code, $lang);

}
