<?php

abstract class InbankBalticsProduct extends AbstractPayment {

    public $object;
    public $api;
    private $uses_granted = 1;
    public $product_code;

    /**
     * InbankBalticsProduct constructor.
     * @param $object_model InbankPayment
     * @throws Exception
     */
    public function __construct($object_model)
    {
        if (is_array($object_model)) {
            $obj = new InbankPayment();
            $obj->hydrate($object_model);
            $object_model = $obj;
        }
        else if (is_int($object_model) && (int)$object_model > 0)
        {
            $object_model = new InbankPayment((int)$object_model);
        }

        if (is_a($object_model, "InbankPayment")) {
            $this->object = $object_model;
        } else {
            throw new Exception('Object must be an instance of InbankPayment');
        }
        $this->product_code = $this->getProductCode();
        $this->api = new InbankBalticsAPI($this->object, $this->getApiUrl());
    }

    /**
     * Returns Product settings form suitable for Wordpress formbuilder
     * @return array[]
     */
    public function productApiForm()
    {

    }

    /**
     * Generates template
     * @param $template
     * @param  false  $force
     */
    public function generateTemplate($template, $force = false)
    {
        PhraseApp::buildTranslationFiles(
            $this->object,
            $this->object->localization_id,
            $this->object->localization_token,
            $template,
            $force
        );
    }

    /**
     * Handle customer payments
     * @param $id_order int Order object id
     */
    public function handlePayment($id_order)
    {

    }


    public function getCalcLargeTemplate($product_price)
    {
        if (!$this->has_large_calc) {
            return false;
        }
        $shop_uuid = $this->object->debug_mode ? $this->object->shop_uuid_debug : $this->object->shop_uuid;
        $calc_link = $this->getEposUrl();
        $calc_link = substr($calc_link, -1) == '/' ? $calc_link : $calc_link.'/';
        $calc_link .= 'calculator/indivy/plan?product_code='
            .$this->getProductCode()
            .'&lang='.PhraseApp::localeTranslator(Context::getContext()->language->iso_code)
            .'&amount='.$product_price
            .'&shop_uuid='.$shop_uuid;

        $placeholders = [
            'calc_link' => $calc_link,
        ];
        $path = INBANK_MODULE_DIR.'product_templates/'.self::INBANK_CALC_LARGE.'.html';
        if ($template = file_get_contents($path)) {
            return self::replacePlaceholders($template, $placeholders);
        }

        return false;
    }

    public function getCurrencySymbolByCode($code)
    {
        $code = strtoupper($code);
        switch($code) {
            case 'EUR':
                return '€';
            case 'PLN':
                return 'zł';
            case 'CZK':
                return 'Kč';
            default:
                return '';
        }
    }

    /**
     * Checks if any single product needs to be updated and updates it
     * @throws Exception
     */
    public static function updatePaymentData()
    {
        //taking one payment
        $yesterday = date("Y-m-d H:i:s", strtotime("yesterday"));
        $sql = 'SELECT id_payment FROM '.InbankFrameworkBridge::getDbPrefix().'inbank_payment WHERE api_last_upd < "'.$yesterday.'" OR api_last_upd IS NULL';
        $payment = (int)InbankFrameworkBridge::getDbValue($sql);

        if ($payment) {
            $payment_obj = new InbankPayment($payment);

            $class_name = $payment_obj->class_name;

            /**
             * @var $inbank_product AbstractPayment|InbankBalticsProduct
             */
            $inbank_product = new $class_name($payment_obj);
            $inbank_product->updateSelf(true);
        }
    }

    /**
     * fills object with data from API. If regenerate templates set to true, saves the object before regenerating
     * @param $regenerate_templates bool should we regenerate templates. If set to true, saves the object before
     *     regenerating
     * @throws Exception
     */
    public function updateSelf($regenerate_templates = false, $product_info = [])
    {
        if (empty($product_info)) {
            $product_info = $this->api->getInfo();
        }
        if (isset($product_info->code) && $product_info->code === $this->getProductCode()) {
            foreach (InbankFrameworkBridge::getLanguagesAvailablePlugin() as $lang) {
                $lang = strtoupper($lang);
                if (!isset($this->object->title[$lang]) || (isset($this->object->title[$lang]) && $this->object->title[$lang] === $this->object->custom_title[$lang])) {
                    $this->object->custom_title[$lang] = $product_info->name;
                }
                $this->object->title[$lang] = $product_info->name;
            }
            $this->object->minimum_order_amount = $product_info->parameters->min_amount;
            $this->object->maximum_order_amount = $product_info->parameters->max_amount;
            $this->object->period = implode(",", $product_info->parameters->period_options);
            $this->object->default_period = $product_info->parameters->default_period;
            $this->object->max_period = !empty($product_info->parameters->period_options) ? max($product_info->parameters->period_options) : $product_info->parameters->default_period;
            $this->object->default_payment_day = !empty($product_info->parameters->default_payment_day) ? (int)$product_info->parameters->default_payment_day : 10;
            $this->object->needs_approval = (int)(isset($product_info->parameters->contract_activation_requires_partner_approval) ? $product_info->parameters->contract_activation_requires_partner_approval : 0);
            $this->object->down_payment = (int)(isset($product_info->parameters->down_payment_enabled) ? $product_info->parameters->down_payment_enabled : 0);
            $this->object->api_last_upd = date('Y-m-d H:i:s');
        }
        if ($regenerate_templates) {
            $this->object->save();
            $this->generateTemplate(self::INBANK_PAYMENT);
            if ($this->object->calc_small) {
                $this->generateTemplate(self::INBANK_CALC_SMALL);
            }
            if ($this->object->down_payment) {
                $this->generateTemplate(self::INBANK_CALC_SMALL_DOWNPAYMENT);
                $this->generateTemplate(self::INBANK_DOWNPAYMENT);
            }
        }
    }

    abstract public static function getSubtype();
}