<?php


class InbankObjectModel
{
    const TYPE_INT = 1;
    const TYPE_BOOL = 2;
    const TYPE_STRING = 3;
    const TYPE_FLOAT = 4;
    const TYPE_DATE = 5;
    const TYPE_HTML = 6;
    const TYPE_NOTHING = 7;
    const TYPE_SQL = 8;
    const TYPE_ARRAY = 3;


    public $id;

    public static $definition = array();

    /**
     * Holds compiled definitions of each InbankObjectModel class.
     * Values are assigned during object initialization.
     *
     * @var array
     */
    protected static $loaded_classes = array();

    /** @var array Contains current object definition. */
    protected $def;

    public function __construct($id = null)
    {
        $class_name = get_class($this);
        if (!isset(InbankObjectModel::$loaded_classes[$class_name])) {
            $this->def = InbankObjectModel::getDefinition($class_name);
        }

        if ($id) {
            $this->load($id, $this, $this->def);
        }
    }

    public static function hydrateCollection($class, array $datas, $id_lang = null)
    {
        if (!class_exists($class)) {
            throw new Exception("Class '$class' not found");
        }

        $collection = array();
        $rows = array();
        if ($datas) {
            $definition = InbankObjectModel::getDefinition($class);
            if (!array_key_exists($definition['primary'], $datas[0])) {
                throw new Exception("Identifier '{$definition['primary']}' not found for class '$class'");
            }

            foreach ($datas as $row) {
                // Get object common properties
                $id = $row[$definition['primary']];
                if (!isset($rows[$id])) {
                    $rows[$id] = $row;
                }

                // Get object lang properties
                if (isset($row['id_lang']) && !$id_lang) {
                    foreach ($definition['fields'] as $field => $data) {
                        if (!empty($data['lang'])) {
                            if (!is_array($rows[$id][$field])) {
                                $rows[$id][$field] = array();
                            }
                            $rows[$id][$field][$row['id_lang']] = $row[$field];
                        }
                    }
                }
            }
        }

        // Hydrate objects
        foreach ($rows as $row) {
            /** @var InbankObjectModel $obj */
            $obj = new $class();
            $obj->hydrate($row, $id_lang);
            $collection[] = $obj;
        }

        return $collection;
    }

    public function hydrate(array $data)
    {
        if (isset($data[$this->def['primary']])) {
            $this->id = $data[$this->def['primary']];
        }
        else if (InbankTools::getIsset($this->def['primary']))
            $this->id = InbankTools::getValue($this->def['primary']);

        foreach ($data as $key => $value) {
            if (property_exists($this, $key)) {
                $this->$key = $value;
            }
        }
    }

    public static function getDefinition($class, $field = null)
    {
        if (is_object($class)) {
            $class = get_class($class);
        }

        $definition = $class::$definition;
        $definition['classname'] = $class;

        if ($field) {
            return isset($definition['fields'][$field]) ? $definition['fields'][$field] : null;
        }

        return $definition;
    }

    public function load($id, $entity, $entity_defs)
    {
        $sql = '';
        $select = 'SELECT *';
        $from = 'FROM ' . InbankFrameworkBridge::getDbPrefix() .$entity_defs['table']. ' a';
        $where = 'WHERE a.`' . $entity_defs['primary'] . '` = ' . (int) $id;
        $join = [];

        // Get lang informations
        if (isset($entity_defs['multilang']) && $entity_defs['multilang']) {
            $join[] = 'LEFT JOIN '. InbankFrameworkBridge::getDbPrefix() . $entity_defs['table'] . '_lang b ON a.`' . $entity_defs['primary'] . '` = b.`' . $entity_defs['primary'].'` ';
        }

        $sql = $select . ' ' . $from . ' ' . implode(' ', $join) . ' ' . $where;
        if ($object_datas = InbankFrameworkBridge::getDbRow($sql)) {
            if (isset($entity_defs['multilang']) && $entity_defs['multilang']) {
                $sql = 'SELECT *
                        FROM `' . InbankFrameworkBridge::getDbPrefix() . $entity_defs['table'] . '_lang`
                        WHERE `' . $entity_defs['primary'] . '` = ' . (int) $id;

                if ($object_datas_lang = InbankFrameworkBridge::getDbResults($sql)) {
                    foreach ($object_datas_lang as $row) {
                        foreach ($row as $key => $value) {
                            if ($key != $entity_defs['primary'] && (array_key_exists($key, $entity_defs['fields'])
                                    || property_exists($entity, $key))) {
                                if (!isset($object_datas[$key]) || !is_array($object_datas[$key])) {
                                    $object_datas[$key] = array();
                                }

                                $object_datas[$key][$row['lang']] = !is_null($value) ? $value : '';
                            }
                        }
                    }
                }
            }

            $primary_field = $entity_defs['primary'];
            $entity->id = (int) $id;
            $entity->$primary_field = (int) $id;

            foreach ($object_datas as $key => $value) {
                if (array_key_exists($key, $entity_defs['fields'])
                    || property_exists($entity, $key))
                {
                    if (isset($entity_defs['fields'][$key]['multi_values']) && $entity_defs['fields'][$key]['multi_values'])
                        $value = array_filter(explode(',', $value), 'strlen');

                    $entity->{$key} = $value;
                } else {
                    unset($object_datas[$key]);
                }
            }
        }
    }

    public function save($auto_date = true)
    {
        $final_data = $final_data_lang = [];
        $errors = [];

        if (!isset($this->id) || empty($this->id))
            $this->id = (int)InbankTools::getValue('id_payment', 0);

        // Automatically fill dates
        if ($auto_date && property_exists($this, 'date_add') && $this->date_add == null) {
            $this->date_add = date('Y-m-d H:i:s');
        }
        if ($auto_date && property_exists($this, 'date_upd')) {
            $this->date_upd = date('Y-m-d H:i:s');
        }

        foreach ($this->def['fields'] as $field_name => $field)
        {
            if (!isset($field['lang']) || (isset($field['lang']) && $field['lang'] === false))
            {
                // SIMPLE FIELD, NOT LANG
                if (isset($this->$field_name) && !is_null($this->$field_name)) {
                    if ($this->validate_field($field_name, $this->$field_name)) {
                        if (isset($field['multi_values']) && $field['multi_values'] && is_array($this->$field_name))
                            $final_data[$field_name] = implode(',', array_filter($this->$field_name));
                        else
                            $final_data[$field_name] = $this->$field_name;
                    } else {
                        $errors[] = sprintf(InbankLocalization::getString(1), $field_name);
                    }
                } else {
                    if (isset($field['lang']) && $field['required']) {
                        $errors[] = sprintf(InbankLocalization::getString(2), $field_name);
                    } else {
                        if (isset($field['default_value'])) {
                            $final_data[$field_name] = $field['default_value'];
                        }
                    }
                }
            }
            else
            {
                // LANG FIELD
                if (isset($this->$field_name) && !is_null($this->$field_name) && is_array($this->$field_name)) {
                    foreach ($this->$field_name as $lang => $field_value)
                    {
                        if ($this->validate_field($field_name, $field_value)) {
                            $final_data_lang[$lang][$field_name] = $field_value;
                        } else {
                            $errors[] = sprintf(InbankLocalization::getString(1), $field_name);
                        }
                    }
                } else {
                    $langs = InbankFrameworkBridge::getLanguagesAvailablePlugin();
                    if ($field['required']) {
                        $errors[] = sprintf(InbankLocalization::getString(2), $field_name);
                    }
                    else {
                        foreach ($langs as $lang) {
                            if (isset($field['default_value'])) {
                                $final_data_lang[$lang][$field_name] = $field['default_value'];
                            } else {
                                $final_data_lang[$lang][$field_name] = '';
                            }
                        }
                    }
                }
            }
        }

        // entity update
        if (!empty($final_data) && empty($errors))
        {
            $table = InbankFrameworkBridge::getDbPrefix().$this->def['table'];
            if ($this->id > 0)
                $result = InbankFrameworkBridge::updateDb($table, $final_data, [$this->def['primary'] => $this->id]);
            else
                $result = InbankFrameworkBridge::insertDb($table, $final_data);

            $final_result = $result !== false;

            if ($final_result)
            {
                if ((int)$this->id == 0)
                    $this->id = InbankFrameworkBridge::getLastInsertedId();

                if (!empty($final_data_lang))
                {
                    foreach ($final_data_lang as $lang => $data) {
                        $table = InbankFrameworkBridge::getDbPrefix() . $this->def['table'] . '_lang';
                        $lang = InbankFrameworkBridge::escapeSql($lang);

                        $data['lang'] = $lang;
                        $data[$this->def['primary']] = $this->id;

                        $exists = InbankFrameworkBridge::getDbValue("SELECT 1 FROM " . $table . ' WHERE ' . $this->def['primary'] . ' = ' . $this->id . ' AND lang = \'' . $lang . '\'');
                        if ($exists) {
                            $result = InbankFrameworkBridge::updateDb($table, $data, [$this->def['primary'] => $this->id, 'lang' => $lang]);
                        } else {
                            $result = InbankFrameworkBridge::insertDb($table, $data);
                        }

                        $final_result = $final_result && ($result !== false);
                    }
                }
            }

            return $final_result;
        }
        else if (!empty($errors))
            return $errors;
        else
            return false;
    }

    public function delete()
    {
        $result = true;

        // Database deletion for multilingual fields related to the object
        if (!empty($this->def['multilang'])) {
            $result &= InbankFrameworkBridge::deleteDb($this->def['table'] . '_lang', [$this->def['primary'] => (int) $this->id]) !== false;
        }

        if ($result) {
            $result &= InbankFrameworkBridge::deleteDb($this->def['table'], [$this->def['primary'] => (int) $this->id]) !== false;
        }

        return (bool)$result;
    }


    public function validate_field($field_name, $value)
    {
        $field_definition = $this->def['fields'][$field_name];
        if (isset($field_definition['validate']))
        {
            $validate_function = $field_definition['validate'];
            if (method_exists('InbankValidate', $validate_function))
                return InbankValidate::$validate_function($value);
        }

        return true;
    }
}