<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 wesite only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 * If you did not receive a copy of the license and are unable to obtain it through the world-wide-web, please send an email
 * to info@inbank.ee so we can send / sell you a licence immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Wordpress to newer
 * versions in the future. If you wish to customize Wordpress for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank <info@inbank.ee>
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankPayment extends InbankObjectModel
{
    /** @var int object identifier */
    public $id_payment;

    /** @var string class identifier */
    public $class_name;

    /** @var string Inbank product identifier */
    public $product_code;

    /** @var string Inbank product identifier */
    public $product_code_debug;

    /** @var int country identifier */
    public $country;

    /** @var bool wether payment is enabled or disabled */
    public $enabled = 1;

    /** @var bool whether debug mode is on */
    public $debug_mode;

    /** @var bool show small calculator */
    public $calc_small = 1;

    /** @var bool show big calculator */
    public $calc_large;

    /** @var string small calculator hook */
    public $calc_small_position;

    /** @var string LIVE api key */
    public $api_key;

    /** @var string DEBUG api key */
    public $api_key_debug;

    /** @var string LIVE api token */
    public $shop_uuid;

    /** @var string DEBUG api token */
    public $shop_uuid_debug;

    /** @var string Localization id */
    public $localization_id;

    /** @var string Localization token */
    public $localization_token;

    /** @var string Title of the payment option */
    public $title;

    /** @var string custom payment option title */
    public $custom_title;

    /** @var string Wether to show logo in the payment options or not */
    public $use_logo = 1;

    /** @var string Logo size used in payment window */
    public $logo_size = 'small';

    /** @var float Minimum order amount for the payment option to show up */
    public $minimum_order_amount;

    /** @var float Maximum order amount for the payment option to show up */
    public $maximum_order_amount;

    /** @var string available periods */
    public $period;

    /** @var int default period */
    public $default_period;

    /** @var int max period */
    public $max_period;

    /** @var int payment date */
    public $default_payment_day;

    /** @var string additional details shown near payment option */
    public $additional_details;

    /** @var string custom additional details */
    public $custom_additional_details;

    /** @var bool does this payment method needs approval from merchant */
    public $needs_approval = 0;

    /** @var bool determines if we need to show the down payment */
    public $down_payment = 0;

    /** @var string granted text */
    public $granted;

    /** @var datetime date time of last update */
    public $api_last_upd = null;

    /** @var string custom logo */
    public $custom_logo = null;

    /** @var string selected categories for restrictions */
    public $restrictions_selected_categories = '';

    /** @var string selected products for restrictions */
    public $restrictions_selected_products = '';

    /** @var string restrictions condition type */
    public $restrictions_condition = 'AND';

    public static $definition = array(
        'table' => 'inbank_payment',
        'primary' => 'id_payment',
        'multilang' => true,
        'fields' => array(
            'class_name' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => true),
            'product_code' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'product_code_debug' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'country' => array('type' => self::TYPE_INT, 'validate' => 'isString', 'required' => true),
            'enabled' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'debug_mode' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'calc_small' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'calc_large' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'calc_small_position' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'api_key' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'api_key_debug' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'shop_uuid' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'shop_uuid_debug' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'localization_id' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'localization_token' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'title' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'custom_title' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'use_logo' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'logo_size' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'minimum_order_amount' => array('type' => self::TYPE_FLOAT, 'validate' => 'isFloat', 'required' => false),
            'maximum_order_amount' => array('type' => self::TYPE_FLOAT, 'validate' => 'isFloat', 'required' => false),
            'period' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false),
            'default_period' => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => false),
            'max_period' => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => false),
            'default_payment_day' => array('type' => self::TYPE_INT, 'validate' => 'isInt', 'required' => false),
            'additional_details' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'custom_additional_details' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'needs_approval' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'down_payment' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => false),
            'granted' => array('type' => self::TYPE_STRING, 'validate' => 'isString', 'required' => false, 'lang' => true),
            'api_last_upd' => array('type' => self::TYPE_DATE, 'validate' => 'isDate', 'required' => false),
            'custom_logo' => array('type' => self::TYPE_STRING, 'required' => false),
            'restrictions_selected_categories' => array('type' => self::TYPE_ARRAY, 'required' => false, 'multi_values' => true),
            'restrictions_selected_products' => array('type' => self::TYPE_ARRAY, 'required' => false, 'multi_values' => true),
            'restrictions_condition' => array('type' => self::TYPE_STRING, 'required' => false),
        ),
    );

    public static function checkForExistanceByClassName($class_name) {
        $table = InbankFrameworkBridge::getDbPrefix().self::$definition['table'];
        $class_name = InbankFrameworkBridge::escapeSql($class_name);
        $sql = "SELECT 1 FROM ".$table." WHERE class_name = '".$class_name."'";
        return (bool)InbankFrameworkBridge::getDbValue($sql);
    }

    public static function checkForExistanceByProductCode($product_code) {
        $table = InbankFrameworkBridge::getDbPrefix().self::$definition['table'];
        $product_code = InbankFrameworkBridge::escapeSql($product_code);
        $sql = "SELECT 1 FROM ".$table." WHERE product_code = '".$product_code."'";
        return (bool)InbankFrameworkBridge::getDbValue($sql);
    }

    public static function checkForDuplicate($product_code, $country, $class_name, $self_id_payment = 0)
    {
        $exists = 'SELECT id_payment FROM '.InbankFrameworkBridge::getDbPrefix().'inbank_payment 
                    WHERE (product_code = "'.InbankFrameworkBridge::escapeSql($product_code).'" 
                    OR product_code_debug = "'.InbankFrameworkBridge::escapeSql($product_code).'") 
                    AND country = "'.$country.'" 
                    AND class_name = "'.InbankFrameworkBridge::escapeSql($class_name).'"';
        if ($self_id_payment !== 0) {
            $exists .= ' AND id_payment <> '.$self_id_payment;
        }

        return (int)InbankFrameworkBridge::getDbValue($exists);
    }

    public static function getPaymentById($id_payment) {
        $table = InbankFrameworkBridge::getDbPrefix().self::$definition['table'];
        $id_payment = InbankFrameworkBridge::escapeSql($id_payment);
        $sql = "SELECT * FROM ".$table." WHERE id_payment = ".$id_payment;
        return InbankFrameworkBridge::getDbRow($sql);
    }

    public function toArray()
    {
        $obj_to_array = [];
        $obj_to_array[self::$definition["primary"]] = $this->id;
        foreach (self::$definition["fields"] as $field => $type) {
            if (is_array($this->$field)) {
                foreach ($this->$field as $lang => $value) {
                    $obj_to_array[$field][$lang] = $value;
                }
            } else {
                $obj_to_array[$field] = $this->$field;
            }
        }

        return $obj_to_array;
    }

    /**
     * @param $payment InbankPayment;
     */
    public static function getProductInfo($payment)
    {
        $errors = [];
        if (isset($payment->id) && $payment->id)
        {
            if ((!empty($payment->api_key) && !empty($payment->shop_uuid)) || (!empty($payment->api_key_debug) && !empty($payment->shop_uuid_debug))) {
                $class_name = $payment->class_name;
                /**
                 * @var $product AbstractPayment|InbankBalticsProduct
                 */
                $product = new $class_name($payment);
                $product_info = $product->api->getInfo();
                if (isset($product_info->code) && $product_info->code == $product->getProductCode()) {
                    if ($product_info->subtype !== $product->subtype) {
                        $errors[] = InbankLocalization::getString(10);
                    }
                    else {
                        //updates payment values from api and generates templates
                        $product->updateSelf(true, $product_info);
                    }
                } else {
                    $payment->enabled = 0;
                    $payment->save();
                    if (isset($product_info->error)) {
                        $errors[] = InbankLocalization::getString(11). ' ' . implode(',', $product_info->error);
                    }
                }
            }
        }

        return empty($errors) ? true : $errors;
    }

    /**
     * Gets all payment methods within the supplied price
     * @param $product_price float product price
     */
    public static function getEligiblePaymentMethods(
        $product_price,
        $country = null,
        $lang = null,
        $hook = null,
        $calc_small = null,
        $calc_large = null
    ) {
        if ($lang === null) {
            $lang = InbankFrameworkBridge::getLocaleIso();
        } else {
            $lang = $lang;
        }
        $product_price = (float)$product_price;
        $sql = 'SELECT ip.*, ipl.title, ipl.additional_details, ipl.custom_title, ipl.custom_additional_details FROM ' . InbankFrameworkBridge::getDbPrefix() . 'inbank_payment ip 
                LEFT JOIN ' . InbankFrameworkBridge::getDbPrefix() . 'inbank_payment_lang ipl ON ip.id_payment = ipl.id_payment 
                AND ipl.lang = \'' . $lang . '\'
                WHERE ip.enabled = 1 AND ' . $product_price . ' >= ip.minimum_order_amount
                AND ' . $product_price . ' <= ip.maximum_order_amount';
        if (country !== null) {
            $sql .= " AND country = '" . $country . "'";
        }
        if ($hook !== null) {
            if ($hook === 'displayProductPriceBlock') {
                $sql .= ' AND (calc_small_position = "' . InbankFrameworkBridge::escapeSql($hook) . '" OR calc_small_position = "")';
            } else {
                $sql .= ' AND calc_small_position = "' . InbankFrameworkBridge::escapeSql($hook) . '"';
            }
        }
        if ($calc_small !== null) {
            $sql .= ' AND calc_small = ' . (int)$calc_small;
        }
        if ($calc_large !== null) {
            $sql .= ' AND calc_large = ' . (int)$calc_large;
        }
        $payments = InbankFrameworkBridge::getDbResults($sql);

        return self::hydrateCollection(self::class, $payments);
    }

    public static function getActivePaymentMethods()
    {
        $sql = 'SELECT * FROM '.InbankFrameworkBridge::getDbPrefix().'inbank_payment WHERE enabled = 1';
        $payments = InbankFrameworkBridge::getDbResults($sql);

        return $payments;
    }

    public static function getActivePaymentMethodsForCalcSmall()
    {
        $sql = 'SELECT * FROM '.InbankFrameworkBridge::getDbPrefix().'inbank_payment WHERE enabled = 1 AND calc_small = 1';
        $payments = InbankFrameworkBridge::getDbResults($sql);

        return $payments;
    }

    public static function getAllPaymentMethods()
    {
        $sql = 'SELECT * FROM '.InbankFrameworkBridge::getDbPrefix().'inbank_payment';
        $payments = InbankFrameworkBridge::getDbResults($sql);

        return $payments;
    }

    public static function updateAllProducts() {
        $options = InbankPayment::getAllPaymentMethods();
        if (!empty($options)) {
            /**
             * @var $option InbankPayment
             */
            foreach ($options as $option) {
                try {
                    $class_name = $option['class_name'];
                    /**
                     * @var $product InbankBalticsProduct|AbstractPayment
                     */
                    if (class_exists($class_name)) {
                        $product = new $class_name((int)$option['id_payment']);
                        $product->updateSelf(true);
                    }
                } catch (Exception $e) {
                    InbankLogs::addLog(0, 'Failed to update products. '.$e->getMessage());
                }
            }
        }
    }

    public static function addCalcHooks()
    {
        $methods = InbankPayment::getActivePaymentMethodsForCalcSmall();
        $hooks = [];

        // gathering hooks
        if (!empty($methods)) {
            foreach ($methods as $method) {
                if (!empty($method['calc_small_position'])) {
                    $hooks[$method['calc_small_position']][] = $method;
                }
            }
        }
        unset($methods);

        if (!empty($hooks))
        {
            foreach ($hooks as $hook_name => $methods)
            {
                add_action($hook_name, function () use ($methods) { InbankPayment::renderCalc($methods); });
            }
        }
    }

    public static function renderCalc($methods)
    {
        if (!isset($GLOBALS['woocommerce'])) {
            $wc_plugin = wp_normalize_path( WP_PLUGIN_DIR ) . '/woocommerce/woocommerce.php';
            if (file_exists($wc_plugin))
                require_once $wc_plugin;
        }

        // updating payment methods
        if (!empty($methods))
            InbankBalticsProduct::updatePaymentData();

        $shop_currency = InbankFrameworkBridge::getCurrency();
        global $product;

        if (!$product || $product->get_id() !== get_the_ID())
            $product = wc_get_product(get_the_ID());

        if (!$product)
            return;

        if (function_exists('wc_get_price_including_tax'))
            $product_price = wc_get_price_including_tax($product);
        else
            $product_price = @$product->get_price_including_tax();

        if (method_exists($product, 'is_type') && method_exists($product, 'get_variation_price') && $product->is_type('variable'))
            $product_price = $product->get_variation_price('max');


        foreach ($methods as $method)
        {
            $class_name = $method['class_name'];

            if (method_exists($class_name, 'getAvailableCurrencies')) {
                $product_available_currencies = $class_name::getAvailableCurrencies();
                if (!in_array($shop_currency, $product_available_currencies)) {
                    continue;
                }
            }

            // continue if product price is less than 0 or equa;
            if (empty($product_price) || $product_price <= 0)
                continue;

            // continue if minimum order amount is less than 0 or equal or product price is lower than required minimum amount
            if ($method['minimum_order_amount'] <= 0 || $product_price < $method['minimum_order_amount'])
                continue;

            // continue if maximum order amount is less than 0 or equal or product price is higher than maximum amount
            if ($method['maximum_order_amount'] <= 0 || $product_price > $method['maximum_order_amount'])
                continue;

            if ( ! InbankPayment::is_calc_available_by_restrictions($product, $method['id_payment']))
                continue;

            $inbank_product = new $class_name($method);
            echo $inbank_product->getCalcSmallTemplate($product_price);
        }
    }


    public static function is_payment_available_by_restrictions($payment_id)
    {
        $restriction_by_category_list_passed = true;
        $restriction_by_product_list_passed = true;

        if (
            is_page( 'checkout' ) ||
            is_checkout() ||
            (
                !empty($_GET['wc-ajax']) && (
                    $_GET['wc-ajax'] == 'update_order_review' ||
                    $_GET['wc-ajax'] == 'checkout'
                )
            )
        )
        {
            $cart = WC()->cart;
            if (!empty($cart))
            {
                $restrictions = new InbankPayment($payment_id);

                $selected_categories = isset($restrictions->restrictions_selected_categories) ? $restrictions->restrictions_selected_categories : '';
                $selected_products = isset($restrictions->restrictions_selected_products) ? $restrictions->restrictions_selected_products : '';
                $restriction_condition = isset($restrictions->restrictions_condition) ? $restrictions->restrictions_condition : '';

                if (!empty($selected_categories) || !empty($selected_products))
                {
                    $products = $cart->get_cart();

                    // checking categories
                    if (!empty($selected_categories)) {
                        foreach ($products as $item => $values) {
                            $id_product = $values['data']->get_id();
                            $product = wc_get_product($id_product);
                            $product_cats_ids = '';

                            if (function_exists('wc_get_product_term_ids')) {
                                $product_cats_ids = wc_get_product_term_ids($product->get_id(), 'product_cat');
                            } else {
                                if (function_exists('get_related_terms')) {
                                    $product_cats_ids = wc_get_product_term_ids($product->get_id(), 'product_cat');
                                }
                            }

                            if (!empty($product_cats_ids))
                            {
                                if (!is_array($product_cats_ids))
                                    $product_cats_ids = explode(',', $product_cats_ids);

                                if (!is_array($selected_categories))
                                    $selected_categories = explode(',', $selected_categories);

                                $product_cats_ids = array_map('intval', $product_cats_ids);
                                $selected_categories = array_map('intval', $selected_categories);
                                $selected_categories = array_diff($selected_categories, [0]);

                                if (!array_intersect($product_cats_ids, $selected_categories))
                                    $restriction_by_category_list_passed = false;
                            }
                            else
                                $restriction_by_category_list_passed = false;

                            if ($restriction_by_category_list_passed === false)
                                break;
                        }
                    }

                    // checking products
                    if (!empty($selected_products)) {
                        foreach ($products as $values) {
                            $id_product = (int)$values['data']->get_id();

                            if (!is_array($selected_products))
                                $selected_products = explode(',', $selected_products);

                            $selected_products = array_map('intval', $selected_products);
                            $selected_products = array_diff($selected_products, [0]);

                            if (!in_array($id_product, $selected_products))
                                $restriction_by_product_list_passed = false;
                        }
                    }
                }
                else
                    return true;
            }
            else
                return false;
        }
        else
            return false;

        if (!isset($restriction_condition) || $restriction_condition !== 'OR')
            return $restriction_by_category_list_passed && $restriction_by_product_list_passed;
        else
            return $restriction_by_category_list_passed || $restriction_by_product_list_passed;
    }


    /**
     * @param WC_Product_Simple $product
     * @param string $payment_id
     * @return bool
     */
    public static function is_calc_available_by_restrictions($product, $payment_id)
    {
        $restriction_by_category_list_passed = true;
        $restriction_by_product_list_passed = true;

        $id_product = $product->get_id();
        $restrictions = new InbankPayment($payment_id);

        $selected_categories = isset($restrictions->restrictions_selected_categories) ? $restrictions->restrictions_selected_categories : '';
        $selected_products = isset($restrictions->restrictions_selected_products) ? $restrictions->restrictions_selected_products : '';
        $restriction_condition = isset($restrictions->restrictions_condition) ? $restrictions->restrictions_condition : '';


        if (!empty($selected_categories) || !empty($selected_products))
        {
            // checking categories
            if (!empty($selected_categories)) {
                $product_cats_ids = '';

                if (function_exists('wc_get_product_term_ids')) {
                    $product_cats_ids = wc_get_product_term_ids($id_product, 'product_cat');
                } else {
                    if (function_exists('get_related_terms')) {
                        $product_cats_ids = wc_get_product_term_ids($id_product, 'product_cat');
                    }
                }

                if (!empty($product_cats_ids))
                {
                    if (!is_array($product_cats_ids))
                        $product_cats_ids = explode(',', $product_cats_ids);

                    if (!is_array($selected_categories))
                        $selected_categories = explode(',', $selected_categories);

                    $product_cats_ids = array_map('intval', $product_cats_ids);
                    $selected_categories = array_map('intval', $selected_categories);
                    $selected_categories = array_diff($selected_categories, [0]);

                    if (!array_intersect($product_cats_ids, $selected_categories))
                        $restriction_by_category_list_passed = false;
                }
                else
                    $restriction_by_category_list_passed = false;
            }

            // checking products
            if (!empty($selected_products))
            {
                if (!is_array($selected_products))
                    $selected_products = explode(',', $selected_products);

                $selected_products = array_map('intval', $selected_products);
                $selected_products = array_diff($selected_products, [0]);

                if (!in_array($id_product, $selected_products))
                    $restriction_by_product_list_passed = false;
            }
        }

        if (!isset($restriction_condition) || $restriction_condition !== 'OR')
            return $restriction_by_category_list_passed && $restriction_by_product_list_passed;
        else
            return $restriction_by_category_list_passed || $restriction_by_product_list_passed;
    }
}