<?php

class InbankInstaller
{
    public static function runInstaller() {
        $res = true;
        $res &= self::createDbTables();
        $res &= self::checkCacheDirectory();

        if ($res)
        {
            $configurator = new InbankConfigurator();
            $configurator->autoConfig();
        }

        return $res;
    }

    private static function createDbTables() {
        global $wpdb;
        $res = true;

        $sqlStatements = self::getQueries();

        foreach ($sqlStatements as $sql)
        {
            $sql = str_replace('PREFIX_', $wpdb->prefix, $sql);
            $sql = str_replace('ENGINE_TYPE', 'INNODB', $sql);
            $res &= $wpdb->query($sql);
        }

        return $res;
    }

    private static function checkCacheDirectory()
    {
        return InbankFrameworkBridge::checkCreateCacheDirectory();
    }

    private static function getQueries()
    {
        return [
          "CREATE TABLE IF NOT EXISTS `PREFIX_inbank_payment` (
            `id_payment` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
            `class_name` VARCHAR(64) NOT NULL,
            `product_code` VARCHAR(128),    
            `product_code_debug` VARCHAR(128),
            `country` VARCHAR(128),
            `enabled` TINYINT(1) DEFAULT 0,
            `debug_mode` TINYINT(1) DEFAULT 0,
            `calc_small` TINYINT(1) DEFAULT 1,
            `calc_small_position` VARCHAR(64),
            `calc_large` TINYINT(1) DEFAULT 0,
            `api_key` VARCHAR(512),
            `api_key_debug` VARCHAR(512),
            `shop_uuid` VARCHAR(512),
            `shop_uuid_debug` VARCHAR(512),
            `localization_id` VARCHAR(512),
            `localization_token` VARCHAR(512),
            `use_logo` TINYINT(1) DEFAULT 1,
            `logo_size` VARCHAR(32),
            `minimum_order_amount` DECIMAL(10,2),
            `maximum_order_amount` DECIMAL(10,2),
            `period` VARCHAR(512),
            `default_period` TINYINT(2),
            `max_period` TINYINT(2),
            `default_payment_day` TINYINT(2),
            `needs_approval` TINYINT(1) DEFAULT 0,
            `down_payment` TINYINT(1) DEFAULT 0,
            `api_last_upd` DATETIME,
            `custom_logo` VARCHAR(255),
            `restrictions_selected_categories` VARCHAR(512),
            `restrictions_selected_products` VARCHAR(512),
            `restrictions_condition` VARCHAR(10),
            PRIMARY KEY (id_payment)
        ) ENGINE = ENGINE_TYPE DEFAULT CHARSET = utf8;",

        "CREATE TABLE IF NOT EXISTS `PREFIX_inbank_payment_lang` (
            `id_payment` INT(11) UNSIGNED NOT NULL,
            `lang` VARCHAR(5),
            `title` VARCHAR(256),
            `custom_title` VARCHAR(256),
            `additional_details` VARCHAR(512),
            `custom_additional_details` VARCHAR(512),
            `granted` VARCHAR(512),
            PRIMARY KEY (`id_payment`, `lang`)
        ) ENGINE = ENGINE_TYPE DEFAULT CHARSET = utf8;",

        "CREATE TABLE IF NOT EXISTS `PREFIX_inbank_detail` (
            `id_detail` INT(11) NOT NULL AUTO_INCREMENT,
            `timestamp` VARCHAR(20),
            `id_order` INT(11) NOT NULL,
            `id_payment` INT(11) NOT NULL,
            `session_uuid` VARCHAR(256) NOT NULL,
            `contract_uuid` VARCHAR(256) NOT NULL,    
            `last_state` VARCHAR(128),
            `error_message` VARCHAR(512) NOT NULL,
            `error_code` VARCHAR(256) NOT NULL,
            PRIMARY KEY (`id_detail`)
        ) ENGINE = ENGINE_TYPE DEFAULT CHARSET = utf8;",

        "CREATE TABLE IF NOT EXISTS `PREFIX_inbank_log` (
            `id_log` INT(11) NOT NULL AUTO_INCREMENT,
            `id_order` INT(11) DEFAULT 0,
            `message` VARCHAR(256) NOT NULL,
            `product_code` VARCHAR(128),
            `endpoint` VARCHAR(128),
            `response` TEXT(2000),
            `type` VARCHAR(32) NOT NULL,
            `date_add` datetime NOT NULL,
            PRIMARY KEY (`id_log`)
        ) ENGINE = ENGINE_TYPE DEFAULT CHARSET = utf8;"
        ];
    }
}