<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 wesite only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 * If you did not receive a copy of the license and are unable to obtain it through the world-wide-web, please send an email
 * to info@inbank.ee so we can send / sell you a licence immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Wordpress to newer
 * versions in the future. If you wish to customize Wordpress for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank <info@inbank.ee>
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankPaymentFormHelper {


    public static function getAllProductCountries() {
        //checking all inbank products and getting their countries
        $countries = [];
        $directory = INBANK_MODULE_DIR.'src/Products/';
        $scanned_directory = array_diff(scandir($directory), array('..', '.', 'index.php'));
        foreach ($scanned_directory as $file)
        {
            $class_name = basename($file, '.php');

            if(method_exists($class_name, 'getAvailableCountries')) {
                $class_countries = $class_name::getAvailableCountries();
                if($class_countries !== null) {
                    if(is_array($class_countries)) {
                        foreach ($class_countries AS $country) {
                            $countries[$country] = $country;
                        }
                    } else if(is_numeric($class_countries)) {
                        $countries[$class_countries] = $class_countries;
                    }
                }
            }
        }

        if(!empty($countries)) {
            return $countries;
        } else {
            return [];
        }
    }


    public static function filterPayments($selected_country) {
        if (empty($selected_country))
            return [];

        $selected_country = strtoupper($selected_country);
        $directory = INBANK_MODULE_DIR.'src/Products/';
        $scanned_directory = array_diff(scandir($directory), array('..', '.', 'index.php'));
        $filtered_classes = [];
        foreach ($scanned_directory as $file)
        {
            $class_name = explode('.', $file)[0];

            if(method_exists($class_name, 'getAvailableCountries')) {
                $class_countries = $class_name::getAvailableCountries();
                if($class_countries !== null) {
                    if(is_array($class_countries) && in_array($selected_country, $class_countries)) {
                        $filtered_classes[$class_name] = self::getProductTitle($class_name);
                    } else if(is_string($class_countries) && $class_countries == $selected_country) {
                        $filtered_classes[$class_name] = self::getProductTitle($class_name);
                    }
                }
            }
        }

        return $filtered_classes;
    }

    public static function getPaymentDataToPrefill($country)
    {
        $api_data = 'SELECT 
                            MAX(api_key) AS api_key, MAX(api_key_debug) AS api_key_debug, MAX(shop_uuid) AS shop_uuid,
                            MAX(shop_uuid_debug) AS shop_uuid_debug
                        FROM '.InbankFrameworkBridge::getDbPrefix().'inbank_payment
                        WHERE country = \''.$country.'\'';

        $api_data2 = 'SELECT 
                            MAX(localization_id) AS localization_id, MAX(localization_token) AS localization_token
                        FROM '.InbankFrameworkBridge::getDbPrefix().'inbank_payment';

        $api_data = InbankFrameworkBridge::getDbRow($api_data);
        $api_data2 = InbankFrameworkBridge::getDbRow($api_data2);

        if (empty($api_data))
            $api_data = [];

        if (empty($api_data2))
            $api_data2 = [];

        return array_merge($api_data, $api_data2);
    }

    private static function getProductTitle($class_name) {

        if(class_exists($class_name)) {
            if(method_exists($class_name, "getDefaultTitle")) {
                return $class_name::getDefaultTitle();
            }
        }
        return $class_name;
    }

    // $args:
    // 0 - options name,
    // 1 - field,
    // 2 - value,
    // 3 - additional text
    // 4 - array for select/radio

    public static function input_url_callback($args)
    {
        if (!isset($args[2]))
            $args[2] = $args[0];

        $html = "<a href=\"{$args[0]}\" class=\"inbank-cron-url\" target=\"_blank\">{$args[2]}</a>";
        //$html .= '<label for="'  . $args[1] . '">&nbsp;'  . $args[3] . '</label>';

        if (!empty($args[1]))
            $html .= '<p class="description inbank-field-description"><i>'.$args[1].'</i></p>';

        echo $html;
    }


    public static function input_textbox_callback($args)
    {
        if( isset( $args[2] ) ) {
            $value = esc_sql( $args[2] );
        }

        if (!empty($args[0]))
            $field_name = 'woocommerce-inbank_'  . $args[0] . '['  . $args[1] . ']';
        else
            $field_name = $args[1];

        $html = '<input type="text" id="'  . $args[1] . '" name="'.$field_name.'" value="' . $value . '" '.(isset($args[4]) ? $args[4] : '').'/>';
        //$html .= '<label for="'  . $args[1] . '">&nbsp;'  . $args[3] . '</label>';

        $html .= '<p class="description inbank-field-description"><i>'.$args[3].'</i></p>';
        echo $html;
    }

    public static function input_textbox_multilang_callback($args)
    {
        $langs = InbankFrameworkBridge::getLanguagesAvailablePlugin();

        $html = '<div class="inbank-flex">';
        if( !is_array( $args[2] ) )
            return '';

        if (!empty($args[0]))
            $field_name = 'woocommerce-inbank_'  . $args[0] . '['  . $args[1] . ']';
        else
            $field_name = $args[1];

        $html .= '<div class="inbank-multilang-input-wrap">';
        foreach ($langs as $lang)
        {
            $field_name_lang = $field_name."[$lang]";
            $value = isset($args[2][$lang]) ? $args[2][$lang] : '';
            $desc_value = isset($args[3][$lang]) ? $args[3][$lang] : '';
            $html .= '<input type="text" class="inbank-input-not-full" id="'  . $args[1] . '_' . $lang . '" name="'.$field_name_lang.'" value="' . $value . '" '. ($args[5] != $lang ? ' style="display: none;" ' : '') .$args[4].'/>';
            $html .= '<p id="'  . $args[1] . '_' . $lang . '_desc" class="description inbank-field-description" '.($args[5] != $lang ? ' style="display: none;" ' : '').'><i>'.$desc_value.'</i></p>';
        }
        $html .= "</div>";

        $html .= '<select id="'  . $args[1] . '_lang_select" class="lang_selector inbank-input-not-full inbank-multilang-select" data-input_field_id="'.$field_name.'" data-desc_field_id="'.$args[1].'">';

        foreach ($langs as $lang) {
            $html .= '<option value="'.$lang.'" '.($args[5] == $lang ? 'selected' : '').'>'.$lang.'</option>';
        }

        $html .= '</select>';
        $html .= '</div>';

        echo $html;
    }

    public static function input_hidden_callback($args)
    {
        $value = '';
        if( isset( $args[2] ) ) {
            $value = esc_sql( $args[2] );
        }

        $is_array = substr($args[1], -2) == '[]';
        if ($is_array)
            $args[1] = substr($args[1], 0, -2);

        if (!empty($args[0]))
            $field_name = 'woocommerce-inbank_'  . $args[0] . '['  . $args[1] . ']';
        else
            $field_name = $args[1];

        if ($is_array)
            $field_name .= '[]';

        $html = '<input type="hidden" name="'.$field_name.'" value="' . $value . '" />';

        echo $html;
    }

    public static function input_multilang_hidden_callback($args)
    {
        if (empty($args[2]))
            return;

        if (!empty($args[0]))
            $field_name = 'woocommerce-inbank_'  . $args[0] . '['  . $args[1] . ']';
        else
            $field_name = $args[1];

        $html = '';
        foreach ($args[2] as $lang => $value)
        {
            $field_name_lang = $field_name."[$lang]";
            $value = $value;
            $html .= '<input type="hidden" id="'  . $args[1] . '_' . $lang . '" name="'.$field_name_lang.'" value="' . $value . '" />';
        }


        echo $html;
    }

    public static function input_checkbox_callback($args)
    {
        if (!empty($args[0]))
            $field_name = 'woocommerce-inbank_'  . $args[0] . '['  . $args[1] . ']';
        else
            $field_name = $args[1];

        $html = '<input type="checkbox" id="'  . $args[1] . '" name="woocommerce-inbank_'  . $args[0] . '['  . $args[1] . ']" value="1" ' . checked( 1, isset( $args[2] ) ? $args[2] : 0, false ) . '/>';
        $html .= '<label for="'  . $args[1] . '">&nbsp;'  . $args[3] . '</label>';

        echo $html;

    }

    public static function input_radio_callback($args)
    {
        $html = '';

        if (is_array($args[4]))
        {
            if (!empty($args[0]))
                $field_name = 'woocommerce-inbank_'  . $args[0] . '['  . $args[1] . ']';
            else
                $field_name = $args[1];

            foreach ($args[4] as $key=>$value)
            {
                $html .= '<input type="radio" id="'.$args[1] . '_' . $key .'" name="'.$field_name.'" value="' . $key . '"' . checked($key,  $args[2], false ) . '/>';
                $html .= '&nbsp;';
                $html .= '<label for="'.$args[1] . '_' . $key .'">' . __( $value, 'woocommerce-inbank' ) . '</label>';
                $html .= '<br />';
            }
        }

        echo $html;
    }

    public static function input_select_callback($args)
    {
        if (!empty($args[0]))
            $field_name = 'woocommerce-inbank_'  . $args[0] . '['  . $args[1] . ']';
        else
            $field_name = $args[1];

        $html = '<select id="'  . $args[1] . '" name="'.$field_name.'" '.(isset($args[5]) ? $args[5] : '').'>';

        if (is_array($args[4]))
        {
            foreach ($args[4] as $key=>$value)
                $html .= '<option value="'.$key.'"' . selected( $args[2], $key, false) . '>' . __( $value, 'woocommerce-inbank' ) . '</option>';
        }

        $html .= '</select>';
        //$html .= '<label for="'  . $args[1] . '">&nbsp;'  . $args[3] . '</label>';
        $html .= '<p class="description inbank-field-description"><i>'.$args[3].'</i></p>';

        echo $html;
    }

    public static function input_file_callback($args)
    {
        if( isset( $args[2] ) ) {
            $value = esc_sql( $args[2] );
        }

        if (!empty($args[0]))
            $field_name = 'woocommerce-inbank_'  . $args[0] . '['  . $args[1] . ']';
        else
            $field_name = $args[1];

        $html = '<input type="file" id="'  . $args[1] . '" name="'.$field_name.'" />';
        //$html .= '<label for="'  . $args[1] . '">&nbsp;'  . $args[3] . '</label>';

        $html .= '<p class="description inbank-field-description"><i>'.$args[3].'</i></p>';
        echo $html;
    }

    public static function input_media_callback($args)
    {
        if( isset( $args[2] ) ) {
            $value = esc_sql( $args[2] );
        }

        if (!empty($args[0]))
            $field_name = 'woocommerce-inbank_'  . $args[0] . '['  . $args[1] . ']';
        else
            $field_name = $args[1];

        // adding media for gallery
        wp_enqueue_media();

        if (!empty($value))
            $custom_logo_url = wp_get_attachment_image_url($value);

        if (!isset($custom_logo_url) || empty($custom_logo_url))
            $custom_logo_url = '';

        $html = '';
        $html .= '<div class="image-preview-wrapper" style="' . (empty($value) ? 'display:none;' : '') . '">
                            <img id="image-preview" src="'.$custom_logo_url.'" width="100" height="100" style="max-height: 100px; width: 100px;">
                        </div>';

        $html .= '<input id="upload_image_button" type="button" class="button" value=" ' . __( 'Upload custom logo' ) . ' " />';
        $html .= '<input id="delete_image_button" type="button" class="button button-link-delete" style="margin-left: 5px; ' . (empty($value) ? 'display:none;' : '') . '" value=" ' . __( 'Delete custom logo' ) . ' " />';
        $html .= '<input type="hidden" name="'.$field_name.'" id="image_attachment_id" value="'.$value.'">';

        echo $html;
    }

    public static function input_textarea_callback($args)
    {
        if( isset( $args[2] ) ) {
            $value = esc_sql( $args[2] );
        }

        if (!empty($args[0]))
            $field_name = 'woocommerce-inbank_'  . $args[0] . '['  . $args[1] . ']';
        else
            $field_name = $args[1];

        $html = '<textarea id="'  . $args[1] . '" name="'.$field_name.'" placeholder="'.$args[3].'" '.$args[4].' rows="4" cols="50">'.$value.'</textarea>';
        //$html .= '<label for="'  . $args[1] . '">&nbsp;'  . $args[3] . '</label>';

        $html .= '<p class="description inbank-field-description"><i>Random</i></p>';
        echo $html;
    }


    public static function input_textarea_multilang_callback($args)
    {
        $langs = InbankFrameworkBridge::getLanguagesAvailablePlugin();


        $html = '<div class="inbank-flex">';
        if( !is_array( $args[2] ) )
            return '';

        if (!empty($args[0]))
            $field_name = 'woocommerce-inbank_'  . $args[0] . '['  . $args[1] . ']';
        else
            $field_name = $args[1];

        $html .= '<div class="inbank-multilang-input-wrap">';
        foreach ($langs as $lang)
        {
            $field_name_lang = $field_name."[$lang]";
            $value = isset($args[2][$lang]) ? $args[2][$lang] : '';
            $desc_value = isset($args[3][$lang]) ? $args[3][$lang] : '';
            $html .= '<textarea id="'  . $args[1] . '_' . $lang . '" name="' . $field_name_lang . '" ' . $args[4] . ($args[5] != $lang ? ' style="display: none;" ' : '') . ' class="inbank-input-not-full" rows="4" cols="50">'.$value.'</textarea>';
            $html .= '<p id="'  . $args[1] . '_' . $lang . '_desc" class="description inbank-field-description" '.($args[5] != $lang ? ' style="display: none;" ' : '').'><i>'.$desc_value.'</i></p>';
        }
        $html .= "</div>";

        $html .= '<select id="'  . $args[1] . '_lang_select" class="lang_selector inbank-input-not-full inbank-multilang-select" data-input_field_id="'.$field_name.'">';
        foreach ($langs as $lang) {
            $html .= '<option value="'.$lang.'" '.($args[5] == $lang ? 'selected' : '').'>'.$lang.'</option>';
        }
        $html .= '</select>';

        //$html .= '<label for="'  . $args[1] . '">&nbsp;'  . $args[3] . '</label>';
        $html .= '</div>';
        echo $html;
    }

    public static function input_label_callback($args)
    {
        if( isset( $args[2] ) ) {
            $value = esc_sql( $args[2] );
        }

        if (!empty($args[0]))
            $field_name = 'woocommerce-inbank_'  . $args[0] . '['  . $args[1] . ']';
        else
            $field_name = $args[1];

        $html = '<label id="'  . $args[1] . '" name="'.$field_name.'" '.$args[4].'>'.$value.'</label>';
        //$html .= '<label for="'  . $args[1] . '">&nbsp;'  . $args[3] . '</label>';

        echo $html;
    }

    public static function product_select_autocomplete($args)
    {
        $name = 'woocommerce-inbank_' . $args[0] . '[' . $args[1] . ']';
        $template = "<li><input type='hidden' name='" . $name . "[]' value='[id_product]' disabled><img data-src='[img_url]' /><label>[title]</label><i class='dashicons-before dashicons-trash'></i></li>";
        $selected_products = $args[2];
        $selected_products_html = '';

        if (!empty($selected_products)) {
            if (!is_array($selected_products)) {
                $selected_products = explode(',', $selected_products);
            }

            $selected_products = array_map('intval', $selected_products);
            $search_results = new WP_Query(array(
                'post_status' => 'publish', // if you don't want drafts to be returned
                'post_type' => 'product',
                'post__in' => $selected_products
            ));

            if ($search_results->have_posts()) {
                while ($search_results->have_posts()) {
                    $search_results->the_post();
                    // shorten the title a little
                    $title = (mb_strlen($search_results->post->post_title) > 50) ? mb_substr($search_results->post->post_title, 0, 49) . '...' : $search_results->post->post_title;
                    $image = get_the_post_thumbnail_url($search_results->post->ID);
                    $selected_products_html .=
                        str_replace('[id_product]', $search_results->post->ID,
                            str_replace('[title]', $title,
                                str_replace('[img_url]', $image,
                                    str_replace('data-src', 'src',
                                        str_replace('disabled', '', $template)
                                    )
                                )
                            )
                        );
                }
            }
        }


        echo '<select class="inbank_product_select"></select>
                <div class="selected_inbank_products">
                    <div style="display:none" class="selected_product_list_item_template">'.$template.'</div>
                    <ul class="selected_product_list">
                        '.$selected_products_html.'
                    </ul>
                </div>';
    }

    public static function post_categories_meta_box( $args ) {
        $defaults = array( 'taxonomy' => 'category' );
        $selected_cats = $args[2]['value'];
        if (!is_array($selected_cats))
            $selected_cats = explode(',', $selected_cats);

        $selected_cats = array_map('intval', $selected_cats);

        $box = $args[2];
        if ( ! isset( $box['args'] ) || ! is_array( $box['args'] ) ) {
            $parsed_args = array();
        } else {
            $parsed_args = $box['args'];
        }
        $parsed_args = wp_parse_args( $parsed_args, $defaults );
        $tax_name    = esc_attr( $parsed_args['taxonomy'] );
        $taxonomy    = get_taxonomy( $parsed_args['taxonomy'] );
        $name = 'woocommerce-inbank_' . $args[0].'['.$args[1].']';

        ob_start();
        $popular_ids = wp_popular_terms_checklist( $tax_name);
        $popular_output = ob_get_contents();
        ob_end_clean();

        $output = '<div id="taxonomy-'.$tax_name.'" class="categorydiv" style="width: fit-content; min-width: 330px;">           
                        <div id="'.$tax_name.'-all" class="tabs-panel">
                            <input type="hidden" name="'.$name.'[]" value="" />
                            <ul id="'.$tax_name.'checklist" data-wp-lists="list:'.$tax_name.'" class="categorychecklist form-no-clear">
                                '.
            wp_terms_checklist(
                0,
                array(
                    'taxonomy'     => $tax_name,
                    'popular_cats' => $popular_ids,
                    'selected_cats' => $selected_cats,
                    'echo' => false
                )
            ).'
                            </ul>
                        </div>
                    </div>';

        $output = str_replace('tax_input[product_cat]', $name, $output);
        echo $output;
    }

    public function sanitize_options( $input ) {

        // Define the array for the updated options
        $output = array();

        // Loop through each of the options sanitizing the data
        foreach( $input as $key => $val ) {

            if( isset ( $input[$key] ) ) {
                $output[$key] = esc_sql( strip_tags( stripslashes( $input[$key] ) ) );
            } // end if

        } // end foreach

        // Return the new collection
        return apply_filters( 'sanitize_options', $output, $input );

    }

    public function validate_inputs( $input ) {

        $input = $this->sanitize_options($input);

        // Create our array for storing the validated options
        $output = array();

        // Loop through each of the incoming options
        foreach( $input as $key => $value ) {

            // Check to see if the current option has a value. If so, process it.
            if( isset( $input[$key] ) ) {

                // Strip all HTML and PHP tags and properly handle quoted strings
                $output[$key] = strip_tags( stripslashes( $input[ $key ] ) );

            } // end if

        } // end foreach

        // Return the array processing any additional functions filtered by this action
        return apply_filters( 'validate_inputs', $output, $input );
    }

    public function validate_api_settings ($input)
    {
        if (isset($input['api_user']) && isset($input['api_pass']))
        {
            $api_class = 'InbankAPI';
            if (class_exists($api_class) && method_exists($api_class, 'testIfAPIUsernameAndPasswordIsValid'))
            {
                $is_valid = call_user_func($api_class.'::testIfAPIUsernameAndPasswordIsValid', $input['api_user'], $input['api_pass']);

                if (!$is_valid)
                {
                    add_settings_error('my_option_notice', 'my_option_notice', __('API username or password is not valid.'), 'error');
                }
            }
        }

        return $input;
    }

    public static function displayWarning($text)
    {
        echo "<div class=\"notice notice-warning is-dismissible below-h2\"> 
                <p><strong>{$text}</strong></p>
            </div>";
    }

    public static function displayConfirmation($text)
    {
        echo "<div class=\"notice notice-success is-dismissible below-h2\"> 
                <p><strong>{$text}</strong></p>
            </div>";
    }
}
