<?php

class InbankTools
{
    public static function getValue($key, $default_value = false)
    {
        if (!isset($key) || empty($key) || !is_string($key)) {
            return false;
        }

        if (getenv('kernel.environment') === 'test' && self::$request instanceof Request) {
            $value = self::$request->request->get($key, self::$request->query->get($key, $default_value));
        } else {
            $value = (isset($_POST[$key]) ? $_POST[$key] : (isset($_GET[$key]) ? $_GET[$key] : $default_value));
        }

        if (is_string($value)) {
            return stripslashes(urldecode(preg_replace('/((\%5C0+)|(\%00+))/i', '', urlencode($value))));
        }

        return $value;
    }

    public static function getIsset($key)
    {
        if (!is_string($key)) {
            return false;
        }

        return isset($_POST[$key]) || isset($_GET[$key]);
    }

    public static function check_prerequisites()
    {
        if ( version_compare( $GLOBALS['wp_version'], '4.7', '<' ) ) {
            return __('Plugin is not compatible with your Wordpress version.', 'woocommerce-inbank');
        }

        if (!class_exists('WC_Payment_Gateway')) {
            return __('Woocommerce is not in activated on your Wordpress.', 'woocommerce-inbank');
        }
        else if (!in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
            return __('Woocommerce is not in activated on your Wordpress.', 'woocommerce-inbank');
        }

        if (empty(InbankFrameworkBridge::getCacheDir())) {
            if (empty(InbankFrameworkBridge::checkCreateCacheDirectory())) {
                return __('No writable cache directory was found. One of these two directories must be writable: /wp-content/cache/inbank/ or /wp-content/plugins/woocommerce-inbank/cache/', 'woocommerce-inbank');
            }
        }

        global $woocommerce;

        if ( version_compare( $woocommerce->version, '2.4.6', "<" ) ) {
            return __('Plugin is not compatible with your Woocommerce version.', 'woocommerce-inbank');
        }

        return true;
    }

    public static function base64_url_encode($input) {
        return strtr(base64_encode($input), '+/=', '._-');
    }

    public static function base64_url_decode($input) {
        return base64_decode(strtr($input, '._-', '+/='));
    }

    public static function multiLang($value)
    {
        $multilangField = [];
        $langs = InbankFrameworkBridge::getLanguagesAvailablePlugin();
        foreach ($langs as $lang) {
            $multilangField[$lang] = $value;
        }

        return $multilangField;
    }

    public static function verifyHmac($hmac, $timestamp, $message, $api_key)
    {
        $calculated_hmac = hash_hmac('sha512', $timestamp.'.'.$message, $api_key);
        if ($hmac === $calculated_hmac) {
            return true;
        }

        return false;
    }
}