<?php

class InbankUpdater
{
    public static function check($module_name, $module_version, $system_version)
    {
        $content = self::getNewestVersion($module_name, $module_version, $system_version);
        $new_version = self::getConfig(strtolower($module_name.'_last_update_check_version'), $module_version);

        if (version_compare($new_version, $module_version, '>'))
        {
            $response_json = self::getConfig(strtolower($module_name.'_last_update_check_response'), '');

            if (!empty($response_json)) {
                $response = json_decode($response_json, true);
                $docs_url = $response['docs_url'];
                $update_url = $response['update_url'];
                $additional_text = base64_decode($response['additional_text_base64']);

                $linkToDownload = $readDocumentation = '';
                if (isset($update_url) && !empty($update_url)) {
                    $linkToDownload = self::getString(1).' - <a target="_blank" href="'.$update_url.'">'.$update_url.'</a>.';
                }

                if (isset($docs_url) && !empty($docs_url)) {
                    $readDocumentation = self::getString(2).' <a target="_blank" href="'.$docs_url.'">'.self::getString(3).'</a>.';
                }

                $content .= self::displayWarning(self::getString(4).' '.$linkToDownload.' '.$readDocumentation.' '.$additional_text);
            }
        }

        return $content;
    }

    private static function getNewestVersion($module_name, $module_version, $system_version)
    {
        try {
            if (self::getConfig(strtolower($module_name.'_last_update_check_date'), 0) < (date("U") - 86400)) {
                self::updateConfig(strtolower($module_name.'_last_update_check_date'), date("U"));

                if (ini_get("allow_url_fopen")) {
                    if (function_exists("file_get_contents")) {
                        $ctx = stream_context_create(array(
                            'http' => array(
                                'timeout' => 10,  // timeout 10 sec
                            ),
                        ));

                        $response_json = file_get_contents('https://developer.inbank.dev/payment-modules/updater/get.php?fw='.self::getModuleFramework().'&m='.$module_name."&v=".self::encrypt($module_version)."&pv=".self::encrypt($system_version)."&h=".self::encrypt(self::getUrl()),
                            false, $ctx);

                        if (!empty($response_json)) {
                            $response = json_decode($response_json, true);

                            if (isset($response['new_version']) && !empty($response['new_version'])) {
                                self::updateConfig(strtolower($module_name.'_last_update_check_version'),
                                    $response['new_version']);
                                self::updateConfig(strtolower($module_name.'_last_update_check_response'),
                                    $response_json);

                                if (version_compare($module_version, $response['new_version'], '=')) {
                                    return self::displayConfirmation(self::getString(5));
                                }
                            }
                        }
                    } else {
                        return self::displayWarning(self::getString(6));
                    }
                } else {
                    return self::displayWarning(self::getString(7));
                }
            }

            return '';
        } catch (Exception $ex) {
            self::updateConfig(strtolower($module_name.'_last_update_check_date'), date("U"));

            return self::displayWarning(self::getString(8).' '.$ex->getMessage());
        }
    }

    private static function encrypt($input)
    {
        return strtr(base64_encode($input), '+/=', '._-');
    }

    // functions witch need to be changed by the framework

    private static function displayConfirmation($text)
    {
        echo "<div class=\"notice notice-success is-dismissible\"> 
                <p><strong>{$text}</strong></p>
            </div>";
    }

    private static function displayWarning($text)
    {
        echo "<div class=\"notice notice-warning is-dismissible\"> 
                <p><strong>{$text}</strong></p>
            </div>";
    }

    private static function getConfig($key, $default_value)
    {
        $options = get_option($key);
        if (empty($options))
            return $default_value;
        else
            return $options;
    }

    private static function updateConfig($key, $value)
    {
        return update_option($key, $value);
    }

    private static function getUrl()
    {
        return get_option('siteurl');
    }

    private static function getModuleFramework()
    {
        return 'wordpress';
    }

    private static function getString($id_string)
    {
        $strings = [
            1 => __('Updated module can be downloaded from', 'woocommerce-inbank'),
            2 => __('More about update', 'woocommerce-inbank'),
            3 => __('read here', 'woocommerce-inbank'),
            4 => __('New module update was found.', 'woocommerce-inbank'),
            5 => __('Successfully checked for updates. No new updates were found. You are using the latest module version.', 'woocommerce-inbank'),
            6 => __('Unable to check for updates. Funcion `file_get_contents` is disabled.', 'woocommerce-inbank'),
            7 => __('Unable to check for updates. Setting `allow_url_fopen` is disabled.', 'woocommerce-inbank'),
            8 => __('Failed to check for updates. Error:', 'woocommerce-inbank'),
        ];

        return isset($strings[$id_string]) ? $strings[$id_string] : '';
    }
}
