<?php


class InbankFrameworkBridge
{
    public static $currency;
    public static $country;
    // ==================================
    // DATABASE
    // ==================================
    public static function getDbPrefix()
    {
        global $wpdb;
        return $wpdb->prefix;
    }

    public static function getDbValue($sql)
    {
        global $wpdb;
        return $wpdb->get_var($sql);
    }

    public static function getDbRow($sql)
    {
        global $wpdb;
        return $wpdb->get_row($sql, ARRAY_A);
    }

    public static function getDbResults($sql)
    {
        global $wpdb;
        return $wpdb->get_results($sql, ARRAY_A);
    }

    public static function getLastInsertedId()
    {
        global $wpdb;
        return (int)$wpdb->insert_id;
    }

    public static function escapeSql($sql)
    {
        return esc_sql($sql);
    }

    /**
     * @param $table string with prefix
     * @param $data array
     * @param $where array
     * @return false|int
     */
    public static function updateDb($table, $data, $where)
    {
        global $wpdb;
        $prefix = InbankFrameworkBridge::getDbPrefix();

        if (substr($table, 0, strlen($prefix)) !== $prefix)
            $table = $prefix.$table;

        return $wpdb->update($table, $data, $where);
    }

    /**
     * @param $table string with prefix
     * @param $data array
     * @return false|int
     */
    public static function insertDb($table, $data)
    {
        global $wpdb;
        $prefix = InbankFrameworkBridge::getDbPrefix();

        if (substr($table, 0, strlen($prefix)) !== $prefix)
            $table = $prefix.$table;

        return $wpdb->insert($table, $data);
    }

    /**
     * @param $table string with prefix
     * @param $data array
     * @return false|int
     */
    public static function deleteDb($table, $where)
    {
        global $wpdb;
        $prefix = InbankFrameworkBridge::getDbPrefix();

        if (substr($table, 0, strlen($prefix)) !== $prefix)
            $table = $prefix.$table;

        return $wpdb->delete($table, $where);
    }

    /**
     * @param $table string with prefix
     * @param $data array
     * @return false|int
     */
    public static function queryDb($sql)
    {
        global $wpdb;
        return $wpdb->query($sql);
    }

    // ==================================
    // OTHER
    // ==================================
    public static function getCacheDir()
    {
        $wp_cache_dir = WP_CONTENT_DIR.'/cache/inbank/';
        $inbank_plugin_cache_dir = INBANK_MODULE_DIR.'/cache/';

        if (is_dir($wp_cache_dir) && is_writable($wp_cache_dir))
            return $wp_cache_dir;

        if (is_dir($inbank_plugin_cache_dir) && is_writable($inbank_plugin_cache_dir))
            return $inbank_plugin_cache_dir;

        return false;
    }

    public static function checkCreateCacheDirectory()
    {
        $success = false;

        if (self::createWordpressDefaultCacheDirectory()) {
                $success = true;
            }

        if (!$success && self::createInbankCacheDirectory()) {
                $success = true;
        }

        return $success;
    }

    public static function createWordpressDefaultCacheDirectory()
    {
        $wp_cache_dir = WP_CONTENT_DIR.'/cache/inbank/';

        if (!is_dir($wp_cache_dir)) {
            if (wp_mkdir_p($wp_cache_dir)) {
                return true;
            }
        }
        return false;
    }

    public static function createInbankCacheDirectory()
    {
        $inbank_plugin_cache_dir = INBANK_MODULE_DIR.'/cache/';

        if (!is_dir($inbank_plugin_cache_dir)) {
            if (wp_mkdir_p($inbank_plugin_cache_dir)) {
                return true;
            }
        }
        return false;
    }

    public static function getLanguagesAvailablePlugin()
    {
        return ['EE', 'LV', 'RU', 'EN', 'PL', 'CZ'];
    }

    public static function getLanguagesInstalledWP()
    {
        return get_available_languages();
    }

    public static function getLocaleIso()
    {
        $determined_locale = get_locale();

        if ( is_admin() ) {
            $determined_locale = get_user_locale();
        }

        $determined_locale = self::iso369_3ToIso369_1($determined_locale);

        if (!empty($determined_locale))
        {
            if (strpos($determined_locale, '_') !== false)
                $separator = '_';
            else
                $separator = '-';
        }
        else
            return 'ee';

        $iso = explode($separator, $determined_locale)[0];
        $iso = strtolower($iso);

        if ($iso == 'et')
            $iso = 'ee';

        if ($iso == 'cs')
            $iso = 'cz';

        return $iso;
    }

    public static function getLocale()
    {
        $determined_locale = get_locale();

        if ( is_admin() ) {
            $determined_locale = get_user_locale();
        }

        if (!empty($determined_locale))
            $determined_locale = str_replace('_', '-', $determined_locale);

        return strtolower($determined_locale);
    }

    public static function getCurrencySymbol($currency = '')
    {
        return get_woocommerce_currency_symbol($currency);
    }

    public static function getCurrency()
    {
        if (!isset(self::$currency) || empty(self::$currency))
        {
            if (function_exists('get_woocommerce_currency'))
                self::$currency = get_woocommerce_currency();
            else
                self::$currency = apply_filters( 'woocommerce_currency', get_option( 'woocommerce_currency' ) );
        }

        return self::$currency;
    }

    public static function getCountry()
    {
        if (!isset(self::$country) || empty(self::$country))
        {
            // checking country restrictions
            $old_wc = version_compare(WC_VERSION, '3.0', '<');
            $customer = WC()->customer;
            self::$country = $old_wc ? $customer->get_country() : $customer->get_billing_country();
            if (empty(self::$country))
                self::$country = WC()->countries->get_base_country();
        }

        return self::$country;
    }

    public static function getImageDir()
    {
        return INBANK_MODULE_DIR.'assets/images/';
    }

    public static function getCountryByIso($country_iso)
    {
        return isset(WC()->countries->get_countries()[$country_iso]) ? WC()->countries->get_countries()[$country_iso] : false;
    }

    /**
     * Checks if checkout is the current page.
     *
     * @return boolean
     */
    public static function is_checkout() {
        $checkout_path    = wp_parse_url(wc_get_checkout_url(), PHP_URL_PATH);
        $current_url_path = wp_parse_url("http://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]", PHP_URL_PATH);

        return (
            $checkout_path !== null
            && $current_url_path !== null
            && trailingslashit($checkout_path) === trailingslashit($current_url_path)
        );
    }

    /**
     * Checks if cart is the current page.
     *
     * @return boolean
     */
    public static function is_cart() {
        $cart_path        = wp_parse_url(wc_get_cart_url(), PHP_URL_PATH);
        $current_url_path = wp_parse_url("http://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]", PHP_URL_PATH);

        return (
            $cart_path !== null
            && $current_url_path !== null
            && trailingslashit($cart_path) === trailingslashit($current_url_path)
        );
    }

    private static function iso369_3ToIso369_1($languageCode)
    {
        $arr = array(
            'aar'=>'aa','abk'=>'ab','ave'=>'ae','afr'=>'af','aka'=>'ak','amh'=>'am','arg'=>'an',
            'ara'=>'ar','asm'=>'as','ava'=>'av','aym'=>'ay','aze'=>'az','bak'=>'ba','bel'=>'be',
            'bul'=>'bg','bis'=>'bi','bam'=>'bm','ben'=>'bn','bod'=>'bo','bod'=>'bo','bre'=>'br',
            'bos'=>'bs','cat'=>'ca','che'=>'ce','cha'=>'ch','cos'=>'co','cre'=>'cr','ces'=>'cs',
            'ces'=>'cs','chu'=>'cu','chv'=>'cv','cym'=>'cy','cym'=>'cy','dan'=>'da','deu'=>'de',
            'deu'=>'de','div'=>'dv','dzo'=>'dz','ewe'=>'ee','ell'=>'el','ell'=>'el','eng'=>'en',
            'epo'=>'eo','spa'=>'es','est'=>'et','eus'=>'eu','eus'=>'eu','fas'=>'fa','fas'=>'fa',
            'ful'=>'ff','fin'=>'fi','fij'=>'fj','fao'=>'fo','fra'=>'fr','fra'=>'fr','fry'=>'fy',
            'gle'=>'ga','gla'=>'gd','glg'=>'gl','grn'=>'gn','guj'=>'gu','glv'=>'gv','hau'=>'ha',
            'heb'=>'he','hin'=>'hi','hmo'=>'ho','hrv'=>'hr','hat'=>'ht','hun'=>'hu','hye'=>'hy',
            'hye'=>'hy','her'=>'hz','ina'=>'ia','ind'=>'id','ile'=>'ie','ibo'=>'ig','iii'=>'ii',
            'ipk'=>'ik','ido'=>'io','isl'=>'is','isl'=>'is','ita'=>'it','iku'=>'iu','jpn'=>'ja',
            'jav'=>'jv','kat'=>'ka','kat'=>'ka','kon'=>'kg','kik'=>'ki','kua'=>'kj','kaz'=>'kk',
            'kal'=>'kl','khm'=>'km','kan'=>'kn','kor'=>'ko','kau'=>'kr','kas'=>'ks','kur'=>'ku',
            'kom'=>'kv','cor'=>'kw','kir'=>'ky','lat'=>'la','ltz'=>'lb','lug'=>'lg','lim'=>'li',
            'lin'=>'ln','lao'=>'lo','lit'=>'lt','lub'=>'lu','lav'=>'lv','mlg'=>'mg','mah'=>'mh',
            'mri'=>'mi','mri'=>'mi','mkd'=>'mk','mkd'=>'mk','mal'=>'ml','mon'=>'mn','mar'=>'mr',
            'msa'=>'ms','msa'=>'ms','mlt'=>'mt','mya'=>'my','mya'=>'my','nau'=>'na','nob'=>'nb',
            'nde'=>'nd','nep'=>'ne','ndo'=>'ng','nld'=>'nl','nld'=>'nl','nno'=>'nn','nor'=>'no',
            'nbl'=>'nr','nav'=>'nv','nya'=>'ny','oci'=>'oc','oji'=>'oj','orm'=>'om','ori'=>'or',
            'oss'=>'os','pan'=>'pa','pli'=>'pi','pol'=>'pl','pus'=>'ps','por'=>'pt','que'=>'qu',
            'roh'=>'rm','run'=>'rn','ron'=>'ro','ron'=>'ro','rus'=>'ru','kin'=>'rw','san'=>'sa',
            'srd'=>'sc','snd'=>'sd','sme'=>'se','sag'=>'sg','sin'=>'si','slk'=>'sk','slk'=>'sk',
            'slv'=>'sl','smo'=>'sm','sna'=>'sn','som'=>'so','sqi'=>'sq','sqi'=>'sq','srp'=>'sr',
            'ssw'=>'ss','sot'=>'st','sun'=>'su','swe'=>'sv','swa'=>'sw','tam'=>'ta','tel'=>'te',
            'tgk'=>'tg','tha'=>'th','tir'=>'ti','tuk'=>'tk','tgl'=>'tl','tsn'=>'tn','ton'=>'to',
            'tur'=>'tr','tso'=>'ts','tat'=>'tt','twi'=>'tw','tah'=>'ty','uig'=>'ug','ukr'=>'uk',
            'urd'=>'ur','uzb'=>'uz','ven'=>'ve','vie'=>'vi','vol'=>'vo','wln'=>'wa','wol'=>'wo',
            'xho'=>'xh','yid'=>'yi','yor'=>'yo','zha'=>'za','zho'=>'zh','zho'=>'zh','zul'=>'zu'
        );
        $languageCode = strtolower($languageCode);
        return (isset($arr[$languageCode])) ? $arr[$languageCode] : $languageCode;
    }
}