<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Wordpress to newer
 * versions in the future. If you wish to customize Wordpress for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankHirePurchasePL extends InbankBalticsProduct
{

    private $class_name = 'InbankHirePurchasePL';
    public $production_api = 'https://api.inbank.pl/partner/v2/';
    public $debug_api = 'https://demo-api.inbank.pl/partner/v2/';
    public $production_epos = 'https://epos.inbank.pl/';
    public $debug_epos = 'https://demo-epos.inbank.pl/';
    public $has_large_calc = true;
    public $default_logo_name = 'inbank-logo.svg';
    public $subtype = 'hire_purchase';
    const INBANK_PAYMENT = 'payment_marketing';
    const INBANK_DOWNPAYMENT = 'downpayment_marketing';

    public static function getDefaultTitle()
    {
        return "Inbank Hire Purchase";
    }

    public static function getAvailableCountries()
    {
        return ['PL'];
    }

    public static function getAvailableCurrencies()
    {
        return ['PLN'];
    }

    public static function getSubtype() {
        return 'hire_purchase';
    }

    public static function getPhraseTemplateVars($template, $additional_info = [])
    {
        $vars = [];
        //$vars['title'] = "woo_plugin_product_hp_marketingMessage";
        $vars['additional_details'] = 'woo_plugin_product_hp_consent';
        $vars['granted'] = 'woo_plugin_granted_text';
        switch ($template) {
            case self::INBANK_DOWNPAYMENT:
            case self::INBANK_PAYMENT:
                $vars['body'] = [
                    'product_granted_text' => 'woo_plugin_product_hp_marketingMessage',
                    'down_payment' => 'woo_plugin_downpayment_message',
                    'selling_point_1' => 'woo_plan_selling_point_one',
                    'selling_point_2' => 'woo_plan_selling_point_two',
                    'selling_point_3' => 'woo_plan_selling_point_three',
                ];
                break;
            case self::INBANK_CALC_SMALL_DOWNPAYMENT:
            case self::INBANK_CALC_SMALL:
                $vars['body'] = [
                    'marketing_message' => 'woo_plugin_product_hp_marketingMessage',
                    'down_payment' => 'woo_plugin_downpayment_message'
                ];
                break;
        }

        return $vars;
    }

    public function getPaymentTemplate($product_price, $calculations = [])
    {
        $template = $this->object->down_payment ? self::INBANK_DOWNPAYMENT : self::INBANK_PAYMENT;
        $path = InbankFrameworkBridge::getCacheDir().$this->object->id.'_'.(int)$this->object->debug_mode.'_'.$template.'_'
            .strtoupper(InbankFrameworkBridge::getLocaleIso()).'.html';
        if (!$calculations) {
            $calculations = $this->api->getCreditCalculation(
                ['amount' => $product_price, 'response_level' => 'advanced']
            );
        }
        $currency_symbol = $this->getCurrencySymbolByCode($calculations->currency);
        $monthly_payment = number_format(round($calculations->payment_amount_monthly, 2), 2, '.', '') . ' ' . $currency_symbol;
        $lang = strtoupper(InbankFrameworkBridge::getLocaleIso());

        $placeholders = [
            'checkbox_image' => INBANK_MODULE_URL.'assets/images/checkbox-inbank.png',
            'monthlyPayment' => $monthly_payment,
            'additional_details' => !empty($this->object->custom_additional_details[$lang]) ? $this->object->custom_additional_details[$lang] : ''
        ];

        if ($this->object->down_payment) {
            $placeholders['min_down_payment'] = number_format(
                (float)(!empty($calculations->down_payment_minimum_amount) ? $calculations->down_payment_minimum_amount : 0),
                2
                ) . ' ' . $currency_symbol;
        }

        if (file_exists($path)) {
            return self::replacePlaceholders(file_get_contents($path), $placeholders);
        } else {
            $this->generateTemplate($template);
            if (file_exists($path)) {
                return self::replacePlaceholders(file_get_contents($path), $placeholders);
            }
        }

        return false;
    }

    public function getCalcSmallTemplate($product_price)
    {
        $template = $this->object->down_payment ? self::INBANK_CALC_SMALL_DOWNPAYMENT : self::INBANK_CALC_SMALL;
        $path = InbankFrameworkBridge::getCacheDir().$this->object->id.'_'.(int)$this->object->debug_mode.'_'.$template.'_'
            .strtoupper(InbankFrameworkBridge::getLocaleIso()).'.html';
        $api_response = $this->api->getCreditCalculation(['amount' => $product_price]);
        if (!isset($api_response->payment_amount_monthly)) {
            return false;
        }
        $currency_symbol = $this->getCurrencySymbolByCode($api_response->currency);
        $placeholders = [
            'monthlyPayment' => number_format(round($api_response->payment_amount_monthly, 2), 2, '.', '') . ' ' . $currency_symbol,
        ];

        if ($this->object->down_payment) {
            $placeholders['min_down_payment'] = number_format(
                (float)(!empty($api_response->down_payment_minimum_amount) ? $api_response->down_payment_minimum_amount : 0),
                2
                ) . ' ' . $currency_symbol;
        }

        if ($this->object->use_logo) {
            $logo = $this->getLogoPath();
            $placeholders['image_link'] = '<img class="inbank_logo_product '.$logo['logo_size'].'-inbank-logo" src="'.$logo['url'].'" style="'.$logo['style'].'"/>';
        } else {
            $placeholders['image_link'] = '';
        }
        if (file_exists($path)) {
            return self::replacePlaceholders(file_get_contents($path), $placeholders);
        } else {
            $this->generateTemplate($template);
            if (file_exists($path)) {
                return self::replacePlaceholders(file_get_contents($path), $placeholders);
            }
        }
    }

    public function renderPaymentForCheckout($product_price, $currency_code, $lang)
    {
        if (!in_array(strtoupper($currency_code), $this::getAvailableCurrencies()))
            return false;

        $calculations = $this->api->getCreditCalculation(
            ['amount' => $product_price, 'currency' => $currency_code]
        );
        if (!isset($calculations->product_code) && !isset($calculations->total_cost)) {
            return false;
        }

        $currency_symbol = $this->getCurrencySymbolByCode($calculations->currency);
        $placeholders = [
            'monthlyPayment' => number_format(round($calculations->payment_amount_monthly, 2), 2, '.', '') . ' '.$currency_symbol,
        ];

        if (!empty($this->object->custom_title)) {
            $title = self::replacePlaceholders($this->object->custom_title, $placeholders);
        } else {
            $title = self::getDefaultTitle();
        }
        $additional_info = $this->getPaymentTemplate($product_price, $calculations);

        return ['title' => $title, 'description' => $additional_info];
    }

    public function productApiForm()
    {
        $form = parent::productApiForm();
        foreach ($form['form']['input'] as $key => $input) {
            if ($input['name'] == 'calc_small') {
                $form['form']['input'][$key]['type'] = 'switch';
            } elseif ($input['name'] == 'calc_small_position') {
                $form['form']['input'][$key]['type'] = 'select';
            }
        }
        return $form;
    }
}
