<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Wordpress to newer
 * versions in the future. If you wish to customize Wordpress for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankIndivyGoEE extends InbankBalticsProduct
{

    private $class_name = 'InbankIndivyGoEE';
    public $production_api = 'https://api.inbank.ee/partner/v2/';
    public $debug_api = 'https://demo-api.inbank.ee/partner/v2/';
    public $default_logo_name = 'inbank-logo.svg';
    public $subtype = 'invoice';

    public static function getDefaultTitle()
    {
        return 'Inbank Pay next month';
    }

    public static function getAvailableCountries()
    {
        return ['EE'];
    }

    public static function getSubtype() {
        return 'invoice';
    }

    public static function getPhraseTemplateVars($template, $additional_info = [])
    {
        $vars = [];
        $vars['title'] = "woo_plugin_product_go_marketingMessage_period";
        $vars['additional_details'] = 'woo_plugin_product_go_consent';
        $vars['granted'] = 'woo_plugin_granted_text';
        switch ($template) {
            case self::INBANK_PAYMENT:
                $vars['body'] = [
                    'down_payment' => 'woo_plugin_downpayment_message',
                    'selling_point_1' => 'woo_go_selling_point_one',
                    'selling_point_2' => 'woo_go_selling_point_two',
                    'selling_point_3' => 'woo_go_selling_point_three',
                ];
                break;
            case self::INBANK_CALC_SMALL:
                $vars['body'] = [
                    'marketing_message' => 'woo_plugin_indivy_product_page_calculation',
                ];
                break;
        }

        return $vars;
    }

    public function getPaymentTemplate($product_price)
    {
        $path = InbankFrameworkBridge::getCacheDir().$this->object->id.'_'.(int)$this->object->debug_mode.'_'
            .self::INBANK_PAYMENT.'_'.strtoupper(InbankFrameworkBridge::getLocaleIso()).'.html';

        $lang = strtoupper(InbankFrameworkBridge::getLocaleIso());

        $placeholders = [
            'checkbox_image' => INBANK_MODULE_URL.'assets/images/checkbox-inbank.png',
            'default_period' => $this->object->period,
            'additional_details' => !empty($this->object->custom_additional_details[$lang]) ? $this->object->custom_additional_details[$lang] : ''
        ];

        if (file_exists($path)) {
            return self::replacePlaceholders(file_get_contents($path), $placeholders);
        } else {
            $template = $this->object->down_payment ? self::INBANK_DOWNPAYMENT : self::INBANK_PAYMENT;
            $this->generateTemplate($template);
            if (file_exists($path)) {
                return self::replacePlaceholders(file_get_contents($path), $placeholders);
            }
        }

        return false;
    }

    public function getCalcSmallTemplate($product_price)
    {
        $path = InbankFrameworkBridge::getCacheDir().$this->object->id.'_'.(int)$this->object->debug_mode.'_'
            .self::INBANK_CALC_SMALL.'_'.strtoupper(InbankFrameworkBridge::getLocaleIso()).'.html';

        $placeholders = [
            'marketing_message' => $this->object->title,
            'payment_date' => date('25.m.Y', strtotime('+1 month'))
        ];

        if ($this->object->use_logo) {
            $logo = $this->getLogoPath();
            $placeholders['image_link'] = '<img class="inbank_logo_product '.$logo['logo_size'].'-inbank-logo" src="'.$logo['url'].'" style="'.$logo['style'].'"/>';
        } else {
            $placeholders['image_link'] = '';
        }

        if (file_exists($path)) {
            return self::replacePlaceholders(file_get_contents($path), $placeholders);
        } else {
            $this->generateTemplate(self::INBANK_CALC_SMALL);
            if (file_exists($path)) {
                return self::replacePlaceholders(file_get_contents($path), $placeholders);
            }
        }

        return false;
    }

    public function renderPaymentForCheckout($product_price, $currency_code, $lang)
    {
        if (!in_array(strtoupper($currency_code), $this::getAvailableCurrencies()))
            return false;

        $placeholders = [
            'payment_date' => date('25.m.Y', strtotime('+1 month'))
        ];

        if (!empty($this->object->custom_title)) {
            $title = self::replacePlaceholders($this->object->custom_title, $placeholders);
        } else {
            $title = self::getDefaultTitle();
        }
        $additional_info = $this->getPaymentTemplate($product_price);

        return ['title' => $title, 'description' => $additional_info];
    }

    public function productApiForm()
    {
        $form = parent::productApiForm();
        foreach ($form['form']['input'] as $key => $input) {
            if ($input['name'] == 'custom_title') {
                $form['form']['input'][$key]['name'] = 'title';
                $form['form']['input'][$key]['disabled'] = true;
                $form['form']['input'][$key]['readonly'] = true;
            }
        }

        return $form;
    }
}
