<?php
/**
 * 2021 Inbank
 *
 * NOTICE OF LICENSE
 *
 * This file is not open source! Each license that you purchased is only available for 1 website only.
 * If you want to use this file on more websites (or projects), you need to purchase additional licenses.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Wordpress to newer
 * versions in the future. If you wish to customize Wordpress for your
 * needs please contact us for extra customization service at an affordable price
 *
 * @author Inbank
 * @copyright  2021 Inbank
 * @license    Valid for 1 website (or project) for each purchase of license
 *
 */

class InbankIndivySliceLV extends InbankBalticsProduct
{

    private $class_name = 'InbankIndivySliceLV';
    public $production_api = 'https://api.inbank.lv/partner/v2/';
    public $debug_api = 'https://demo-api.inbank.lv/partner/v2/';
    public $has_large_calc = false;
    public $default_logo_name = 'inbank-logo.svg';
    public $subtype = 'slice';
    const INBANK_PAYMENT = 'payment_marketing';


    public static function getDefaultTitle()
    {
        return 'Inbank Split into parts';
    }

    public static function getAvailableCountries()
    {
        return ['LV'];
    }

    public static function getSubtype() {
        return 'slice';
    }

    public static function getPhraseTemplateVars($template, $additional_info = [])
    {
        $default_period = 0;
        $vars = array();
        if (isset($additional_info['default_period'])) {
            $default_period = $additional_info['default_period'];
        }
        if ($default_period == 3) {
            $vars['title'] = 'woo_plugin_product_slice_titleMessage';
        } else {
            $vars['title'] = 'woo_plugin_product_slice_titleMessage';
        }
        $vars['granted'] = 'woo_plugin_granted_text';
        $vars['additional_details'] = 'woo_plugin_product_slice_consent';

        switch ($template) {
            case self::INBANK_PAYMENT:
                $vars['body'] = [
                    'down_payment' => 'woo_plugin_downpayment_message',
                    'selling_point_1' => 'woo_slice_selling_point_one',
                    'selling_point_2' => 'woo_slice_selling_point_two',
                    'selling_point_3' => 'woo_slice_selling_point_three',
                ];
                if ($default_period == 3) {
                    $vars['body']['product_granted_text'] = 'woo_plugin_indivy_parts_3month_product_page';
                } else {
                    $vars['body']['product_granted_text'] = 'woo_plugin_indivy_parts_over3month_product_page';
                }
                break;
            case self::INBANK_CALC_SMALL:
                if ($default_period == 3) {
                    $vars['body'] = [
                        'marketing_message' => 'woo_plugin_indivy_parts_3month_product_page',
                    ];
                } else {
                    $vars['body'] = [
                        'marketing_message' => 'woo_plugin_indivy_parts_over3month_product_page',
                    ];
                }
                break;
        }

        return $vars;
    }

    public function getPaymentTemplate($product_price, $calculations = [])
    {
        if (!$calculations) {
            $calculations = $this->api->getCreditCalculation([
                'amount' => $product_price,
                'response_level' => 'payment_schedule',
                'period' => $this->object->default_period
            ]);
        }
        $path = InbankFrameworkBridge::getCacheDir().$this->object->id.'_'.(int)$this->object->debug_mode.'_'
            .self::INBANK_PAYMENT.'_'.strtoupper(InbankFrameworkBridge::getLocaleIso()).'.html';

        $currency_symbol = $this->getCurrencySymbolByCode($calculations->currency);
        if ($this->object->use_logo) {
            $logo = $this->getLogoPath();
            $image_link = '<img class="inbank_logo_product '.$logo['logo_size'].'-inbank-logo" src="'.$logo['url'].'" style="'.$logo['style'].'"/>';
        }
        else {
            $image_link = '';
        }

        if ($this->object->default_period == 3) {
            if (isset($calculations->payment_schedule[1]->total_payment)) {
                $placeholders = [
                    'checkbox_image' => INBANK_MODULE_URL.'assets/images/checkbox-inbank.png',
                    'image_link' => $image_link,
                    'payment_one' => number_format($calculations->payment_schedule[1]->total_payment, 2).' '.$currency_symbol,
                    'payment_two' => number_format($calculations->payment_schedule[2]->total_payment, 2).' '.$currency_symbol,
                    'payment_three' => number_format($calculations->payment_schedule[3]->total_payment, 2).' '.$currency_symbol,
                    'default_period' => $this->object->default_period
                ];
            } else return false;
        } else {
            $placeholders = [
                'checkbox_image' => INBANK_MODULE_URL.'assets/images/checkbox-inbank.png',
                'image_link' => $image_link,
                'default_period' => $this->object->default_period,
                'payment_one' => number_format($calculations->payment_schedule[1]->total_payment, 2).' '.$currency_symbol,
            ];
        }
        $lang = strtoupper(InbankFrameworkBridge::getLocaleIso());
        $placeholders['additional_details'] = !empty($this->object->custom_additional_details[$lang]) ? $this->object->custom_additional_details[$lang] : '';

        if (file_exists($path)) {
            return self::replacePlaceholders(file_get_contents($path), $placeholders);
        } else {
            $template = $this->object->down_payment ? self::INBANK_DOWNPAYMENT : self::INBANK_PAYMENT;
            $this->generateTemplate($template);
            if (file_exists($path)) {
                return self::replacePlaceholders(file_get_contents($path), $placeholders);
            }
        }

        return false;
    }

    public function renderPaymentForCheckout($product_price, $currency_code, $lang)
    {
        if (!in_array(strtoupper($currency_code), $this::getAvailableCurrencies()))
            return false;

        $calculations = $this->api->getCreditCalculation([
            'amount' => $product_price,
            'currency' => $currency_code,
            'response_level' => 'payment_schedule',
            'period' => $this->object->default_period
        ]);
        if (!isset($calculations->product_code) && !isset($calculations->total_cost)) {
            return false;
        }

        $currency_symbol = $this->getCurrencySymbolByCode($calculations->currency);

        if ($this->object->use_logo) {
            $logo = $this->getLogoPath();
            $image_link = '<img class="inbank_logo_product '.$logo['logo_size'].'-inbank-logo" src="'.$logo['url'].'" style="'.$logo['style'].'"/>';
        }
        else {
            $image_link = '';
        }
        if ($this->object->default_period == 3) {
            if (isset($calculations->payment_schedule[1]->total_payment)) {
                
                $placeholders = [
                    'checkbox_image' => INBANK_MODULE_URL.'assets/images/checkbox-inbank.png',
                    'image_link' => $image_link,
                    'payment_one' => number_format($calculations->payment_schedule[1]->total_payment, 2).' '.$currency_symbol,
                    'payment_two' => number_format($calculations->payment_schedule[2]->total_payment, 2).' '.$currency_symbol,
                    'payment_three' => number_format($calculations->payment_schedule[3]->total_payment, 2).' '.$currency_symbol,
                    'default_period' => $this->object->default_period
                ];
            } else return false;
        } else {
            $placeholders = [
                'checkbox_image' => INBANK_MODULE_URL.'assets/images/checkbox-inbank.png',
                'image_link' => $image_link,
                'default_period' => $this->object->default_period,
            ];
        }

        if (!empty($this->object->custom_title)) {
            $title = self::replacePlaceholders($this->object->custom_title, $placeholders);
        } else {
            $title = self::getDefaultTitle();
        }
        $additional_info = $this->getPaymentTemplate($product_price, $calculations);

        return ['title' => $title, 'description' => $additional_info];
    }

    public function getCalcSmallTemplate($product_price)
    {
        $path = InbankFrameworkBridge::getCacheDir().$this->object->id.'_'.(int)$this->object->debug_mode.'_'
            .self::INBANK_CALC_SMALL.'_'.strtoupper(InbankFrameworkBridge::getLocaleIso()).'.html';

        $api_response = $this->api->getCreditCalculation([
            'amount' => $product_price,
            'response_level' => 'payment_schedule',
            'period' => $this->object->default_period
        ]);
        if ($api_response == false || isset($api_response->message)) {
            return false;
        }
        $currency_symbol = $this->getCurrencySymbolByCode($api_response->currency);
        if ($this->object->default_period == 3) {
            if (!isset($api_response->payment_schedule[1]->total_payment)) {
                return false;
            }
            $placeholders = [
                'payment_one' => number_format($api_response->payment_schedule[1]->total_payment, 2) . ' ' . $currency_symbol,
                'payment_two' => number_format($api_response->payment_schedule[2]->total_payment, 2) . ' ' . $currency_symbol,
                'payment_three' => number_format($api_response->payment_schedule[3]->total_payment, 2) . ' ' . $currency_symbol,
            ];
        } else {
            $placeholders = [
                'default_period' => $this->object->default_period,
                'payment_one' => number_format($api_response->payment_schedule[1]->total_payment, 2) . ' ' . $currency_symbol,
            ];
        }
        if ($this->object->use_logo) {
            $logo = $this->getLogoPath();
            $placeholders['image_link'] = '<img class="inbank_logo_product '.$logo['logo_size'].'-inbank-logo" src="'.$logo['url'].'" style="'.$logo['style'].'"/>';
        } else {
            $placeholders['image_link'] = '';
        }
        
        if (file_exists($path)) {
            return self::replacePlaceholders(file_get_contents($path), $placeholders);
        } else {
            $this->generateTemplate(self::INBANK_CALC_SMALL);
            if (file_exists($path)) {
                return self::replacePlaceholders(file_get_contents($path), $placeholders);
            }
        }

        return false;
    }

    public function getCalcLargeTemplate($product_price) {

    }

    /**
     * @param $detail_object InbankDetail
     *
     * @return Object json decoded object
     */
    public function checkPaymentStatus($detail_object)
    {
        return $this->api->getSessionDetails($detail_object->session_uuid);
    }

    public function productApiForm()
    {
        $form = parent::productApiForm();
        foreach ($form['form']['input'] as $key => $input) {
            if ($input['name'] == 'custom_title') {
                $form['form']['input'][$key]['name'] = 'title';
                $form['form']['input'][$key]['disabled'] = true;
                $form['form']['input'][$key]['readonly'] = true;
            }
        }

        return $form;
    }
}
