<?php

/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @since             1.0.0
 * @package           Woocommerce_Inbank
 *
 * @wordpress-plugin
 * Plugin Name:       Pay Later Platform by Inbank
 * Description:       Increase your revenue by turning visitors into customers. Pay later payment solutions from Inbank make your goods more affordable for your customers whilst you are still selling at full price.
 * Version:           2.1.2
 * Author:            Inbank
 * Author URI:        https://inbank.ee
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       woocommerce-inbank
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'WOOCOMMERCE_INBANK_VERSION', '2.1.2' );

define( 'INBANK_MODULE_DIR', plugin_dir_path( __FILE__ ));
define( 'INBANK_MODULE_URL', plugin_dir_url( __FILE__ ));
define( 'INBANK_USE_TIPPY', true);
define( 'WOOCOMMERCE_INBANK_DEBUG', false);

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-woocommerce-inbank-activator.php
 */
function activate_woocommerce_inbank() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-woocommerce-inbank-activator.php';
	Woocommerce_Inbank_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-woocommerce-inbank-deactivator.php
 */
function deactivate_woocommerce_inbank() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-woocommerce-inbank-deactivator.php';
	Woocommerce_Inbank_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_woocommerce_inbank' );
register_deactivation_hook( __FILE__, 'deactivate_woocommerce_inbank' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-woocommerce-inbank.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_woocommerce_inbank() {
	$plugin = new Woocommerce_Inbank();
	$plugin->run();

}

run_woocommerce_inbank();
